@@ -4,13 +4,15 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include "hd.h"
+#include "hd_int.h"
 
 static int get_probe_flags(int, char **, hd_data_t *);
-static void progress(char *, char *);
+static void progress2(char *, char *);
 
 // ##### temporary solution, fix it later!
 str_list_t *read_file(char *file_name, unsigned start_line, unsigned lines);
@@ -34,7 +36,7 @@ static int hw_items = 0;
 int braille_install_info(hd_data_t *hd_data);
 int x11_install_info(hd_data_t *hd_data);
 int oem_install_info(hd_data_t *hd_data);
-int dump_packages(hd_data_t *hd_data);
+void dump_packages(hd_data_t *hd_data);
 
 void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item);
 void do_hw_multi(hd_data_t *hd_data, FILE *f, hd_hw_item_t *hw_items);
@@ -43,17 +45,21 @@ void do_test(hd_data_t *hd_data);
 void help(void);
 void dump_db_raw(hd_data_t *hd_data);
 void dump_db(hd_data_t *hd_data);
+void do_chroot(hd_data_t *hd_data, char *dir);
+void ask_db(hd_data_t *hd_data, char *query);
 
 
 struct {
   unsigned db_idx;
   unsigned separate:1;
+  char *root;
 } opt;
 
 struct option options[] = {
   { "special", 1, NULL, 1 },
-  { "help", 1, NULL, 'h' },
+  { "help", 0, NULL, 'h' },
   { "debug", 1, NULL, 'd' },
+  { "version", 0, NULL, 400 },
   { "log", 1, NULL, 'l' },
   { "packages", 0, NULL, 'p' },
   { "test", 0, NULL, 300 },
@@ -65,6 +71,10 @@ struct option options[] = {
   { "dump-db", 1, NULL, 306 },
   { "dump-db-raw", 1, NULL, 307 },
   { "separate", 0, NULL, 308 },
+  { "root", 1, NULL, 309 },
+  { "db", 1, NULL, 310 },
+  { "only", 1, NULL, 311 },
+  { "listmd", 0, NULL, 312 },
   { "cdrom", 0, NULL, 1000 + hw_cdrom },
   { "floppy", 0, NULL, 1000 + hw_floppy },
   { "disk", 0, NULL, 1000 + hw_disk },
@@ -117,12 +127,17 @@ struct option options[] = {
   { "hotplug_ctrl", 0, NULL, 1000 + hw_hotplug_ctrl },
   { "zip", 0, NULL, 1000 + hw_zip },
   { "pppoe", 0, NULL, 1000 + hw_pppoe },
-  { "dsl", 0, NULL, 1000 + hw_pppoe },
+  { "dsl", 0, NULL, 1000 + hw_dsl },
   { "wlan", 0, NULL, 1000 + hw_wlan },
+  { "redasd", 0, NULL, 1000 + hw_redasd },
+  { "block", 0, NULL, 1000 + hw_block },
+  { "tape", 0, NULL, 1000 + hw_tape },
+  { "vbe", 0, NULL, 1000 + hw_vbe },
   { "all", 0, NULL, 2000 },
   { "reallyall", 0, NULL, 2001 },
   { "smp", 0, NULL, 2002 },
   { "arch", 0, NULL, 2003 },
+  { "uml", 0, NULL, 2004 },
   { }
 };
 
@@ -135,12 +150,11 @@ int main(int argc, char **argv)
   hd_data_t *hd_data;
   hd_t *hd;
   FILE *f = NULL;
-  long l = 0;
   int i;
   unsigned first_probe = 1;
 
   hd_data = calloc(1, sizeof *hd_data);
-  hd_data->progress = isatty(1) ? progress : NULL;
+  hd_data->progress = isatty(1) ? progress2 : NULL;
   hd_data->debug=~(HD_DEB_DRIVER_INFO | HD_DEB_HDDB);
 
   for(i = 0; i < argc; i++) {
@@ -221,6 +235,26 @@ int main(int argc, char **argv)
           opt.separate = 1;
           break;
 
+        case 309:
+          opt.root = optarg;
+          break;
+
+        case 310:
+          ask_db(hd_data, optarg);
+          break;
+
+        case 311:
+          if(*optarg) add_str_list(&hd_data->only, optarg);
+          break;
+
+        case 312:
+          hd_data->flags.list_md = 1;
+          break;
+
+        case 400:
+          printf("%s\n", hd_version());
+	  break;
+
         case 1000 ... 1100:
           if(hw_items < (int) (sizeof hw_item / sizeof *hw_item) - 1)
             hw_item[hw_items++] = i - 1000;
@@ -230,6 +264,7 @@ int main(int argc, char **argv)
         case 2001:
         case 2002:
         case 2003:
+        case 2004:
           if(hw_items < (int) (sizeof hw_item / sizeof *hw_item) - 1)
             hw_item[hw_items++] = i;
           break;
@@ -243,7 +278,16 @@ int main(int argc, char **argv)
     if(!hw_items && is_short) hw_item[hw_items++] = 2000;	/* all */
 
     if(hw_items >= 0 || showconfig || saveconfig) {
-      if(*log_file) f = fopen(log_file, "w+");
+      if(*log_file) {
+        if(!strcmp(log_file, "-")) {
+          f = fdopen(1, "w");
+        }
+        else {
+          f = fopen(log_file, "w+");
+        }
+      }
+
+      if(opt.root) do_chroot(hd_data, opt.root);
 
       if(opt.separate || hw_items <= 1) {
         for(i = 0; i < hw_items; i++) {
@@ -314,13 +358,22 @@ int main(int argc, char **argv)
     deb = hd_data->debug;
     argc -= i; argv += i;
 
+    if(opt.root && first_probe) do_chroot(hd_data, opt.root);
+
     hd_scan(hd_data);
     if(hd_data->progress) printf("\r%64s\r", "");
 
     first_probe = 0;
   } while(argc);
 
-  if(*log_file) f = fopen(log_file, "w+");
+  if(*log_file) {
+    if(!strcmp(log_file, "-")) {
+      f = fdopen(1, "w");
+    }
+    else {
+      f = fopen(log_file, "w+");
+    }
+  }
 
   if((hd_data->debug & HD_DEB_SHOW_LOG) && hd_data->log) {
     if(*log_file) {
@@ -334,8 +387,6 @@ int main(int argc, char **argv)
     );
   }
 
-  if(f) l = ftell(f);
-
   for(hd = hd_data->hd; hd; hd = hd->next) {
     hd_dump_entry(hd_data, hd, f ? f : stdout);
   }
@@ -358,6 +409,7 @@ int main(int argc, char **argv)
     if(!strcmp(list, "keyboard")) i = hw_keyboard;
     if(!strcmp(list, "sound")) i = hw_sound;
     if(!strcmp(list, "isdn")) i = hw_isdn;
+    if(!strcmp(list, "dsl")) i = hw_dsl;
     if(!strcmp(list, "modem")) i = hw_modem;
     if(!strcmp(list, "storage_ctrl")) i = hw_storage_ctrl;
     if(!strcmp(list, "network_ctrl")) i = hw_network_ctrl;
@@ -391,7 +443,7 @@ int main(int argc, char **argv)
 void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
 {
   hd_t *hd, *hd0;
-  int smp = -1, i;
+  int smp = -1, uml = 0, i;
   char *s, *t;
   enum boot_arch b_arch;
   enum cpu_arch c_arch;
@@ -420,6 +472,10 @@ void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
       }
       break;
 
+    case 2004:
+      uml = hd_is_uml(hd_data);
+      break;
+
     default:
       hd0 = hd_list(hd_data, hw_item, 1, NULL);
   }
@@ -540,6 +596,9 @@ void do_hw(hd_data_t *hd_data, FILE *f, hd_hw_item_t hw_item)
 
     fprintf(f ? f : stdout, "Arch: %s/%s\n", s, t);
   }
+  else if(hw_item == 2004) {
+    fprintf(f ? f : stdout, "UML: %s\n", uml ? "yes" : "no");
+  }
   else {
     if(is_short) {
       /* always to stdout */
@@ -1249,6 +1308,7 @@ void help()
     "  --short        just a short listing\n"
     "  --log logfile  write info to logfile\n"
     "  --debug level  set debuglevel\n"
+    "  --version      show libhd version\n"
     "  --dump-db n    dump hardware data base, 0: external, 1: internal\n"
     "  --hw_item      probe for hw_item\n"
     "  hw_item is one of:\n"
@@ -1256,7 +1316,7 @@ void help()
     "    mouse, joystick, keyboard, chipcard, sound, isdn, modem, storage-ctrl,\n"
     "    netcard, printer, tv, dvb, scanner, braille, sys, bios, cpu, partition,\n"
     "    usb-ctrl, usb, pci, isapnp, ide, scsi, bridge, hub, memory, smp, pppoe,\n"
-    "    pcmcia, pcmcia-ctrl, wlan, zip, all, reallyall\n\n"
+    "    pcmcia, pcmcia-ctrl, wlan, zip, dsl, all, reallyall\n\n"
     "  Note: debug info is shown only in the log file. (If you specify a\n"
     "  log file the debug level is implicitly set to a reasonable value.)\n"
   );
@@ -1302,6 +1362,18 @@ int get_probe_flags(int argc, char **argv, hd_data_t *hd_data)
       continue;
     }
 
+    t = "only=";
+    if(!strncmp(s, t, strlen(t))) {
+      add_str_list(&hd_data->only, s + strlen(t));
+      continue;
+    }
+
+    t = "root=";
+    if(!strncmp(s, t, strlen(t))) {
+      opt.root = s + strlen(t);
+      continue;
+    }
+
     k = 1;
     if(*s == '+')
       s++;
@@ -1326,7 +1398,7 @@ int get_probe_flags(int argc, char **argv, hd_data_t *hd_data)
 /*
  * A simple progress function.
  */
-void progress(char *pos, char *msg)
+void progress2(char *pos, char *msg)
 {
   if(!test) printf("\r%64s\r", "");
   printf("> %s: %s", pos, msg);
@@ -1439,7 +1511,7 @@ char *get_x11i()
   char buf[256], *s, *t;
   static char x11i[64] = { };
 
-  if(!*x11i) return x11i;
+  if(*x11i) return x11i;
 
   if((f = fopen("/proc/cmdline", "r"))) {
     if(fgets(buf, sizeof buf, f)) {
@@ -1513,7 +1585,6 @@ char *get_xserver(hd_data_t *hd_data, char **version, char **busid, driver_info_
     }
   }
 
-
   if(*display) return display;
 
   if(c == 0) c = '4';	/* default to XF 4, if nothing else is known  */
@@ -1559,7 +1630,7 @@ int x11_install_info(hd_data_t *hd_data)
     }
     /* don't free di */
   }
-  
+
   xserver = get_xserver(hd_data, &version, &busid, &x11_driver);
 
   switch(hd_mac_color(hd_data)) {
@@ -1639,6 +1710,7 @@ int x11_install_info(hd_data_t *hd_data)
   return 0;
 }
 
+
 char *xserver3map[] =
 {
 #ifdef __i386__
@@ -1669,7 +1741,7 @@ char *xserver3map[] =
 };
 
 
-int dump_packages(hd_data_t *hd_data)
+void dump_packages(hd_data_t *hd_data)
 {
   str_list_t *sl;
   int i;
@@ -1687,10 +1759,9 @@ int dump_packages(hd_data_t *hd_data)
   for(; sl; sl = sl->next) {
     printf("%s\n", sl->str);
   }
-
-  return 0;
 }
 
+
 struct x11pack {
   struct x11pack *next;
   char *pack;
@@ -1793,3 +1864,97 @@ void dump_db(hd_data_t *hd_data)
   hddb_dump(hd_data->hddb2[opt.db_idx], stdout);
 }
 
+
+void do_chroot(hd_data_t *hd_data, char *dir)
+{
+  int i;
+
+  i = chroot(dir);
+  ADD2LOG("chroot %s: %s\n", dir, i ? strerror(errno) : "ok");
+
+  if(!i) chdir("/");
+}
+
+
+void ask_db(hd_data_t *hd_data, char *query)
+{
+  hd_t *hd;
+  driver_info_t *di;
+  str_list_t *sl, *query_sl;
+  unsigned tag = 0, u, cnt;
+  char buf[256];
+
+  setenv("hwprobe", "-all", 1);
+  hd_scan(hd_data);
+
+  hd = add_hd_entry(hd_data, __LINE__, 0);
+
+  query_sl = hd_split(' ', query);
+
+  for(sl = query_sl; sl; sl = sl->next) {
+    if(!strcmp(sl->str, "pci")) { tag = TAG_PCI; continue; }
+    if(!strcmp(sl->str, "usb")) { tag = TAG_USB; continue; }
+    if(!strcmp(sl->str, "pnp")) { tag = TAG_EISA; continue; }
+    if(!strcmp(sl->str, "isapnp")) { tag = TAG_EISA; continue; }
+    if(!strcmp(sl->str, "special")) { tag = TAG_SPECIAL; continue; }
+    if(!strcmp(sl->str, "pcmcia")) { tag = TAG_PCMCIA; continue; }
+
+    if(sscanf(sl->str, "vendor=%i%n", &u, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->vendor.id = MAKE_ID(tag, u);
+      continue;
+    }
+
+    if(sscanf(sl->str, "vendor=%3s%n", buf, &cnt) >= 1 && !sl->str[cnt]) {
+      u = name2eisa_id(buf);
+      if(u) hd->vendor.id = u;
+      tag = TAG_EISA;
+      continue;
+    }
+
+    if(sscanf(sl->str, "device=%i%n", &u, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->device.id = MAKE_ID(tag, u);
+      continue;
+    }
+
+    if(sscanf(sl->str, "subvendor=%i%n", &u, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->sub_vendor.id = MAKE_ID(tag, u);
+      continue;
+    }
+
+    if(sscanf(sl->str, "subvendor=%3s%n", buf, &cnt) >= 1 && !sl->str[cnt]) {
+      u = name2eisa_id(buf);
+      if(u) hd->sub_vendor.id = u;
+      tag = TAG_EISA;
+      continue;
+    }
+
+    if(sscanf(sl->str, "subdevice=%i%n", &u, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->sub_device.id = MAKE_ID(tag, u);
+      continue;
+    }
+
+    if(sscanf(sl->str, "revision=%i%n", &u, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->revision.id = u;
+      continue;
+    }
+
+    if(sscanf(sl->str, "serial=%255s%n", buf, &cnt) >= 1 && !sl->str[cnt]) {
+      hd->serial = new_str(buf);
+      continue;
+    }
+
+  }
+
+  free_str_list(query_sl);
+
+  hddb_add_info(hd_data, hd);
+
+  for(di = hd->driver_info; di; di = di->next) {
+    if(di->any.type == di_module && di->module.modprobe) {
+      for(sl = di->module.names; sl; sl = sl->next) {
+        printf("%s%c", sl->str, sl->next ? ' ' : '\n');
+      }
+    }
+  }
+}
+
