@@ -14,6 +14,8 @@
 #include "hd.h"
 #include "hd_int.h"
 #include "bios.h"
+#include "smbios.h"
+#include "klog.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  * bios info
@@ -21,7 +23,7 @@
  * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  */
 
-#if defined(__i386__) || defined (__x86_64__) || defined(__ia64__)
+#if defined(__i386__) || defined (__x86_64__)
 
 #ifndef LIBHD_TINY
 static struct {
@@ -65,9 +67,7 @@ static void read_memory(memory_range_t *mem);
 #ifndef LIBHD_TINY
 static void dump_memory(hd_data_t *hd_data, memory_range_t *mem, int sparse, char *label);
 static void get_pnp_support_status(memory_range_t *mem, bios_info_t *bt);
-static void get_smbios_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt);
-static char *get_string(str_list_t *sl, int index);
-static void parse_smbios(hd_data_t *hd_data, bios_info_t *bt);
+static void smbios_get_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt);
 static void get_fsc_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt);
 static void add_panel_info(hd_data_t *hd_data, bios_info_t *bt);
 static void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt);
@@ -122,6 +122,7 @@ void hd_scan_bios(hd_data_t *hd_data)
   hd_res_t *res;
 #endif
   struct stat sbuf;
+  str_list_t *sl;
 
   if(!hd_probe_feature(hd_data, pr_bios)) return;
 
@@ -314,7 +315,7 @@ void hd_scan_bios(hd_data_t *hd_data)
 
   if(hd_data->bios_rom.data) {
     get_pnp_support_status(&hd_data->bios_rom, bt);
-    get_smbios_info(hd_data, &hd_data->bios_rom, bt);
+    smbios_get_info(hd_data, &hd_data->bios_rom, bt);
     get_fsc_info(hd_data, &hd_data->bios_rom, bt);
     add_panel_info(hd_data, bt);
     add_mouse_info(hd_data, bt);
@@ -348,7 +349,7 @@ void hd_scan_bios(hd_data_t *hd_data)
 
   if(bt->smp.ok && bt->smp.mpconfig) {
     mem.start = bt->smp.mpconfig;
-    mem.size = 1 << 10;
+    mem.size = 1 << 16;
     mem.data = NULL;
     read_memory(&mem);
     parse_mpconfig(hd_data, &mem, &bt->smp);
@@ -379,12 +380,20 @@ void hd_scan_bios(hd_data_t *hd_data)
     }
   }
 
-  if(hd_probe_feature(hd_data, pr_bios_vbe)) {
+  if(hd_probe_feature(hd_data, pr_bios_vesa)) {
     PROGRESS(4, 0, "vbe");
 
     vbe = &bt->vbe;
     vbe->ok = 0;
 
+    if(!hd_data->klog) read_klog(hd_data);
+    for(sl = hd_data->klog; sl; sl = sl->next) {
+      if(sscanf(sl->str, "<6>PCI: Using configuration type %u", &u) == 1) {
+        hd_data->pci_config_type = u;
+        ADD2LOG("  klog: pci config type %u\n", hd_data->pci_config_type);
+      }
+    }
+
     get_vbe_info(hd_data, vbe);
 
     if(vbe->ok) {
@@ -396,6 +405,8 @@ void hd_scan_bios(hd_data_t *hd_data)
       hd->base_class.id = bc_framebuffer;
       hd->sub_class.id = sc_fb_vesa;
 
+      hd_set_hw_class(hd, hw_vbe);
+
 #if 0
       hd->detail = new_mem(sizeof *hd->detail);
       hd->detail->type = hd_detail_bios;
@@ -455,11 +466,6 @@ void hd_scan_bios(hd_data_t *hd_data)
     }
   }
 
-#if 0
-  if(hd_probe_feature(hd_data, pr_bios_32)) {
-  }
-#endif
-
   PROGRESS(5, 0, "32");
 
   mem = hd_data->bios_rom;
@@ -506,18 +512,14 @@ void read_memory(memory_range_t *mem)
   mem->data = new_mem(mem->size);
   fd = -1;
   if(
-    !(
 #ifdef BIOS_TEST
-      (fd = open(s ? s : DEV_MEM, O_RDONLY)) >= 0 &&
+    (fd = open(s ? s : DEV_MEM, O_RDONLY)) >= 0 &&
 #else
-      (fd = open(DEV_MEM, O_RDONLY)) >= 0 &&
+    (fd = open(DEV_MEM, O_RDONLY)) >= 0 &&
 #endif
-      lseek(fd, mem->start, SEEK_SET) >= 0 &&
-      (unsigned) read(fd, mem->data, mem->size) == mem->size
-    )
+    lseek(fd, mem->start, SEEK_SET) >= 0
   ) {
-    mem->data = free_mem(mem->data);
-    mem->size = mem->start = 0;
+    read(fd, mem->data, mem->size);
   }
 
   if(fd >= 0) close(fd);
@@ -585,7 +587,7 @@ unsigned char crc(unsigned char *mem, unsigned len)
 }
 
 
-void get_smbios_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
+void smbios_get_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
 {
   unsigned u, u1, u2, ok, hlen = 0, ofs;
   unsigned addr = 0, len = 0, scnt;
@@ -612,7 +614,7 @@ void get_smbios_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
 
   bt->smbios_ver = (mem->data[u + 6] << 8) + mem->data[u + 7];
 
-  hd_data->smbios = free_smbios_list(hd_data->smbios);
+  hd_data->smbios = smbios_free(hd_data->smbios);
 
   memory.start = mem->start + u;
   memory.size = hlen;
@@ -637,7 +639,7 @@ void get_smbios_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
     type = memory.data[ofs];
     slen = memory.data[ofs + 1];
     if(ofs + slen > len || slen < 4) break;
-    sm = add_smbios_entry(&hd_data->smbios, new_mem(sizeof *sm));
+    sm = smbios_add_entry(&hd_data->smbios, new_mem(sizeof *sm));
     sm->any.type = type;
     sm->any.data_len = slen;
     sm->any.data = new_mem(slen);
@@ -682,184 +684,7 @@ void get_smbios_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
 
   memory.data = free_mem(memory.data);
 
-  parse_smbios(hd_data, bt);
-}
-
-
-char *get_string(str_list_t *sl, int index)
-{
-  if(!sl || !index) return NULL;
-
-  for(; sl; sl = sl->next, index--) {
-    if(index == 1) return new_str(sl->str && *sl->str ? sl->str : NULL);
-  }
-
-  return NULL;
-}
-
-
-void parse_smbios(hd_data_t *hd_data, bios_info_t *bt)
-{
-  hd_smbios_t *sm;
-  str_list_t *sl;
-  int cnt, data_len;
-  unsigned char *sm_data;
-  unsigned u, v;
-
-  if(!hd_data->smbios) return;
-
-  for(cnt = 0, sm = hd_data->smbios; sm; sm = sm->next, cnt++) {
-    sm_data = sm->any.data;
-    data_len = sm->any.data_len;
-    sl = sm->any.strings;
-    switch(sm->any.type) {
-      case sm_biosinfo:
-        if(data_len >= 0x12) {
-          sm->biosinfo.vendor = get_string(sl, sm_data[4]);
-          sm->biosinfo.version = get_string(sl, sm_data[5]);
-          sm->biosinfo.date = get_string(sl, sm_data[8]);
-          memcpy(&sm->biosinfo.features, sm_data + 0xa, 8);
-        }
-        if(data_len >= 0x13) {
-          sm->biosinfo.xfeatures = sm_data[0x12];
-        }
-        if(data_len >= 0x14) {
-          sm->biosinfo.xfeatures |= sm_data[0x13] << 8;
-        }
-        break;
-
-      case sm_sysinfo:
-        if(data_len >= 8) {
-          sm->sysinfo.manuf = get_string(sl, sm_data[4]);
-          sm->sysinfo.product = get_string(sl, sm_data[5]);
-          sm->sysinfo.version = get_string(sl, sm_data[6]);
-          sm->sysinfo.serial = get_string(sl, sm_data[7]);
-        }
-        if(data_len >= 0x19) {
-          sm->sysinfo.wake_up = sm_data[0x18];
-        }
-        break;
-
-      case sm_boardinfo:
-        if(data_len >= 8) {
-          sm->boardinfo.manuf = get_string(sl, sm_data[4]);
-          sm->boardinfo.product = get_string(sl, sm_data[5]);
-          sm->boardinfo.version = get_string(sl, sm_data[6]);
-          sm->boardinfo.serial = get_string(sl, sm_data[7]);
-        }
-        break;
-
-      case sm_chassis:
-        if(data_len >= 6) {
-          sm->chassis.manuf = get_string(sl, sm_data[4]);
-          sm->chassis.ch_type = sm_data[5] & 0x7f;
-        }
-        break;
-
-      case sm_processor:
-        if(data_len >= 0x20) {
-          sm->processor.socket = get_string(sl, sm_data[4]);
-          sm->processor.manuf = get_string(sl, sm_data[7]);
-          sm->processor.version = get_string(sl, sm_data[0x10]);
-          sm->processor.voltage = sm_data[0x11];
-          if(sm->processor.voltage & 0x80) {
-            sm->processor.voltage &= 0x7f;
-          }
-          else {
-            switch(sm->processor.voltage) {
-              case 0x01:
-                sm->processor.voltage = 50;
-                break;
-              case 0x02:
-                sm->processor.voltage = 33;
-                break;
-              case 0x04:
-                sm->processor.voltage = 29;
-                break;
-              default:
-                sm->processor.voltage = 0;
-            }
-          }
-          sm->processor.ext_clock = *(uint16_t *) (sm_data + 0x12);
-          sm->processor.max_speed = *(uint16_t *) (sm_data + 0x14);
-          sm->processor.current_speed = *(uint16_t *) (sm_data + 0x16);
-          sm->processor.status = sm_data[0x18];
-          sm->processor.upgrade = sm_data[0x19];
-        }
-        break;
-
-      case sm_onboard:
-        if(data_len >= 4) {
-          u = data_len - 4;
-          if(!(u & 1)) {
-            u >>= 1;
-            if(u > sizeof sm->onboard.descr / sizeof *sm->onboard.descr) {
-              u = sizeof sm->onboard.descr / sizeof *sm->onboard.descr;
-            }
-            for(v = 0; v < u; v++) {
-              sm->onboard.descr[v] = get_string(sl, sm_data[4 + (v << 1) + 1]);
-              sm->onboard.dtype[v] = sm_data[4 + (v << 1)];
-            }
-          }
-        }
-        break;
-
-      case sm_lang:
-        if(data_len >= 0x16) {
-          sm->lang.current = get_string(sl, sm_data[0x15]);
-        }
-        break;
-
-      case sm_memarray:
-        if(data_len >= 0x0b) {
-          sm->memarray.ecc = sm_data[6];
-          memcpy(&sm->memarray.max_size, sm_data + 0x7, 4);
-        }
-        break;
-
-      case sm_memdevice:
-        if(data_len >= 0x15) {
-          sm->memdevice.eccbits = *(uint16_t *) (sm_data + 8);
-          sm->memdevice.width = *(uint16_t *) (sm_data + 0xa);
-          if(sm->memdevice.width == 0xffff) sm->memdevice.width = 0;
-          if(sm->memdevice.eccbits == 0xffff) sm->memdevice.eccbits = 0;
-          if(sm->memdevice.eccbits >= sm->memdevice.width) {
-            sm->memdevice.eccbits -= sm->memdevice.width;
-          }
-          else {
-            sm->memdevice.eccbits = 0;
-          }
-          sm->memdevice.size = *(uint16_t *) (sm_data + 0xc);
-          if(sm->memdevice.size == 0xffff) sm->memdevice.size = 0;
-          if((sm->memdevice.size & 0x8000)) {
-            sm->memdevice.size &= 0x7fff;
-          }
-          else {
-            sm->memdevice.size <<= 10;
-          }
-          sm->memdevice.form = sm_data[0xe];
-          sm->memdevice.location = get_string(sl, sm_data[0x10]);
-          sm->memdevice.bank = get_string(sl, sm_data[0x11]);
-          sm->memdevice.type1 = sm_data[0x12];
-          sm->memdevice.type2 = sm_data[0x13] + (sm_data[0x14] << 8);
-        }
-        if(data_len >= 0x17) {
-          sm->memdevice.speed = sm_data[0x15] + (sm_data[0x16] << 8);
-        }
-        break;
-
-      case sm_mouse:
-        if(data_len >= 7) {
-          sm->mouse.mtype = sm_data[4];
-          sm->mouse.interface = sm_data[5];
-          sm->mouse.buttons = sm_data[6];
-        }
-        break;
-
-      default:
-	break;
-    }
-  }
+  smbios_parse(hd_data);
 }
 
 
@@ -987,10 +812,6 @@ void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
   unsigned compat_vend, compat_dev, bus;
   char *vendor, *name, *type;
   hd_smbios_t *sm;
-  static char *mice[] = {
-    NULL, "Touch Pad" /* normally "Other" */, NULL, "Mouse",
-    "Track Ball", "Track Point", "Glide Point", "Touch Pad"
-  };
 
   if(bt->mouse.compat_vend || !hd_data->smbios) return;
 
@@ -1009,7 +830,7 @@ void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
       compat_vend = compat_dev = bus = 0;
       type = NULL;
       
-      switch(sm->mouse.interface) {
+      switch(sm->mouse.interface.id) {
         case 4:	/* ps/2 */
         case 7:	/* bus mouse (dell notebooks report this) */
           bus = bus_ps2;
@@ -1017,7 +838,9 @@ void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
           compat_dev = MAKE_ID(TAG_SPECIAL, sm->mouse.buttons == 3 ? 0x0007 : 0x0006);
           break;
       }
-      type = mice[sm->mouse.mtype < sizeof mice / sizeof *mice ? sm->mouse.mtype : 0];
+      type = sm->mouse.mtype.name;
+      if(sm->mouse.mtype.id == 1) type = "Touch Pad";	/* Why??? */
+      if(sm->mouse.mtype.id == 2) type = NULL;		/* "Other" */
     }
   }
 
@@ -1026,7 +849,7 @@ void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
   if(!type) {
     if(!strcmp(vendor, "Sony Corporation") && strstr(name, "PCG-") == name) {
       bus = bus_ps2;
-      type = mice[7];
+      type = "Touch Pad";
       compat_vend = MAKE_ID(TAG_SPECIAL, 0x0200);
       compat_dev = MAKE_ID(TAG_SPECIAL, 0x0006);
     }
@@ -1142,6 +965,10 @@ void parse_mpconfig(hd_data_t *hd_data, memory_range_t *mem, smp_info_t *smp)
       if(len + u > xcfg_len) len = xcfg_len - u;
       hexdump(&hd_data->log, 1, len, mem->data + cfg_len + u);
       ADD2LOG("\n");
+      if(len < 2) {
+        ADD2LOG("  oops: invalid record lenght\n");
+        break;
+      }
     }
   }
 }
@@ -1154,7 +981,7 @@ int get_bios32_info(hd_data_t *hd_data, memory_range_t *mem, bios32_info_t *bios
 
   if(mem->size < 0x10) return 0;
 
-  for(u = ok = 0; u <= mem->size - 0x10; u++) {
+  for(u = ok = 0; u <= mem->size - 0x10; u += 0x10) {
     if(*(unsigned *) (mem->data + u) == 0x5f32335f) {	/* "_32_" */
       addr = *(unsigned *) (mem->data + u + 4);
       len = mem->data[u + 9];
@@ -1179,4 +1006,4 @@ int get_bios32_info(hd_data_t *hd_data, memory_range_t *mem, bios32_info_t *bios
 #endif		/* !defined(LIBHD_TINY) */
 
 
-#endif /* defined(__i386__) || defined (__x86_64__) || defined (__ia64__) */
+#endif /* defined(__i386__) || defined (__x86_64__) */
