@@ -30,7 +30,7 @@
 static void read_cpuinfo(hd_data_t *hd_data);
 static void dump_cpu_data(hd_data_t *hd_data);
 
-#ifdef __i386__
+#if defined(__i386__) || defined(__x86_64__)
 static inline unsigned units_per_cpu();
 #endif
 #ifdef __ia64__
@@ -298,9 +298,7 @@ void read_cpuinfo(hd_data_t *hd_data)
         if(*features) {
           for(t0 = features; (t = strsep(&t0, " ")); ) {
             add_str_list(&ct->features, t);
-#ifdef __i386__
             if(!strcmp(t, "ht")) ct->units = units_per_cpu();
-#endif
           }
         }
 
@@ -497,7 +495,7 @@ void dump_cpu_data(hd_data_t *hd_data)
 }
 
 
-#ifdef __i386__
+#if defined(__i386__) || defined(__x86_64__)
 inline unsigned units_per_cpu()
 {
   unsigned u;
@@ -518,34 +516,34 @@ inline unsigned units_per_cpu()
 #endif
 
 
-#if 1
 #ifdef __ia64__
 
 /*
  * IA64 SMP detection code
  */
 
-#define ACPI_RSDP_SIG "RSD PTR "
-#define ACPI_RSDT_SIG "RSDT"
-#define ACPI_SAPIC_SIG "SPIC"
+#define PAGE_OFFSET(addr) ((uintptr_t) (addr) & (getpagesize () - 1))
 
-#define PAGE_OFFSET(addr) ((uintptr_t)(addr) & (getpagesize() - 1))
+typedef struct
+{
+  uint8_t type;
+  uint8_t length;
+} __attribute__ ((packed)) acpi_table_entry_header;
 
-struct acpi20_rsdp
+struct acpi20_table_rsdp
 {
   char signature[8];
   uint8_t checksum;
   char oem_id[6];
   uint8_t revision;
-  uint32_t rsdt;
-  uint32_t lenth;
-  unsigned long xsdt;
+  uint32_t rsdt_address;
+  uint32_t length;
+  unsigned long xsdt_address;
   uint8_t ext_checksum;
   uint8_t reserved[3];
 } __attribute__ ((packed));
-typedef struct acpi20_rsdp acpi20_rsdp_t;
 
-struct acpi_desc_table_hdr
+struct acpi_table_header
 {
   char signature[4];
   uint32_t length;
@@ -554,112 +552,81 @@ struct acpi_desc_table_hdr
   char oem_id[6];
   char oem_table_id[8];
   uint32_t oem_revision;
-  uint32_t creator_id;
-  uint32_t creator_revision;
-} __attribute__ ((packed));
-typedef struct acpi_desc_table_hdr acpi_desc_table_hdr_t;
+  char asl_compiler_id[4];
+  uint32_t asl_compiler_revision;
+};
 
 #define ACPI_XSDT_SIG "XSDT"
-struct acpi_xsdt
+struct acpi_table_xsdt
 {
-  acpi_desc_table_hdr_t header;
-  unsigned long entry_ptrs[0];
+  struct acpi_table_header header;
+  unsigned long entry[0];
 } __attribute__ ((packed));
-typedef struct acpi_xsdt acpi_xsdt_t;
 
-#define ACPI_MADT_SIG "APIC"
-struct acpi_madt {
-  acpi_desc_table_hdr_t header;
+#define ACPI_MADT_SIG "ACPI"
+struct acpi_table_madt
+{
+  struct acpi_table_header header;
   uint32_t lapic_address;
-  uint32_t flags;
+  struct
+  {
+    uint32_t pcat_compat:1;
+    uint32_t reserved:31;
+  } flags;
 } __attribute__ ((packed));
-typedef struct acpi_madt acpi_madt_t;
 
-#define ACPI20_ENTRY_LOCAL_SAPIC                7
+#define ACPI_MADT_LSAPIC 7
 
-struct acpi20_entry_lsapic
+struct acpi_table_lsapic
 {
-  uint8_t type;
-  uint8_t length;
-  uint8_t acpi_processor_id;
+  acpi_table_entry_header header;
+  uint8_t acpi_id;
   uint8_t id;
   uint8_t eid;
   uint8_t reserved[3];
-  uint32_t flags;
+  struct
+  {
+    uint32_t enabled:1;
+    uint32_t reserved:31;
+  } flags;
 } __attribute__ ((packed));
-typedef struct acpi20_entry_lsapic acpi20_entry_lsapic_t;
-
-struct acpi_rsdt
-{
-  acpi_desc_table_hdr_t header;
-  uint8_t reserved[4];
-  unsigned long entry_ptrs[0];
-} __attribute__ ((packed));
-typedef struct acpi_rsdt acpi_rsdt_t;
-
-struct acpi_sapic
-{
-  acpi_desc_table_hdr_t header;
-  uint8_t reserved[4];
-  unsigned long interrupt_block;
-} __attribute__ ((packed));
-typedef struct acpi_sapic acpi_sapic_t;
-
-/* SAPIC structure types */
-#define ACPI_ENTRY_LOCAL_SAPIC         0
-
-struct acpi_entry_lsapic
-{
-  uint8_t type;
-  uint8_t length;
-  uint16_t acpi_processor_id;
-  uint16_t flags;
-  uint8_t id;
-  uint8_t eid;
-} __attribute__ ((packed));
-typedef struct acpi_entry_lsapic acpi_entry_lsapic_t;
-
-/* Local SAPIC flags */
-#define LSAPIC_ENABLED                (1<<0)
-#define LSAPIC_PERFORMANCE_RESTRICTED (1<<1)
-#define LSAPIC_PRESENT                (1<<2)
 
 /*
  * Map an ACPI table into virtual memory
  */
-static acpi_desc_table_hdr_t *
+static struct acpi_table_header *
 acpi_map_table (int mem, unsigned long addr, char *signature)
 {
   /* mmap header to determine table size */
-  acpi_desc_table_hdr_t *table = NULL;
+  struct acpi_table_header *table = NULL;
   unsigned long offset = PAGE_OFFSET (addr);
   uint8_t *mapped = mmap (NULL,
-			  sizeof (acpi_desc_table_hdr_t) + offset,
+			  sizeof (struct acpi_table_header) + offset,
 			  PROT_READ,
 			  MAP_PRIVATE,
 			  mem,
 			  (unsigned long) addr - offset);
-  table = (acpi_desc_table_hdr_t *) (mapped != MAP_FAILED
-				     ? mapped + offset
-				     : NULL);
+  table = (struct acpi_table_header *) (mapped != MAP_FAILED
+					? mapped + offset
+					: NULL);
   if (table)
     {
       if (memcmp (table->signature, signature, sizeof (table->signature)))
 	{
 	  munmap ((char *) table - offset,
-		  sizeof (acpi_desc_table_hdr_t) + offset);
+		  sizeof (struct acpi_table_header) + offset);
 	  return NULL;
 	}
       {
 	/* re-mmap entire table */
 	unsigned long size = table->length;
 	munmap ((uint8_t *) table - offset,
-		sizeof (acpi_desc_table_hdr_t) + offset);
+		sizeof (struct acpi_table_header) + offset);
 	mapped = mmap (NULL, size + offset, PROT_READ, MAP_PRIVATE, mem,
 		       (unsigned long) addr - offset);
-	table = (acpi_desc_table_hdr_t *) (mapped != MAP_FAILED
-					   ? mapped + offset
-					   : NULL);
+	table = (struct acpi_table_header *) (mapped != MAP_FAILED
+					      ? mapped + offset
+					      : NULL);
       }
     }
   return table;
@@ -669,7 +636,7 @@ acpi_map_table (int mem, unsigned long addr, char *signature)
  * Unmap an ACPI table from virtual memory
  */
 static void
-acpi_unmap_table (acpi_desc_table_hdr_t * table)
+acpi_unmap_table (struct acpi_table_header * table)
 {
   if (table)
     {
@@ -678,171 +645,97 @@ acpi_unmap_table (acpi_desc_table_hdr_t * table)
     }
 }
 
-#if 0
-/*
- * Locate the RSDP
- */
-static unsigned long
-acpi_find_rsdp (int mem,
-		unsigned long base, unsigned long size, acpi20_rsdp_t *rsdp)
-{
-  unsigned long addr = 0;
-  uint8_t *mapped = mmap (NULL, size, PROT_READ, MAP_PRIVATE, mem, base);
-  if (mapped != MAP_FAILED)
-    {
-      uint8_t *i;
-      for (i = mapped; i < mapped + size; i += 16)
-	{
-	  acpi20_rsdp_t *r = (acpi20_rsdp_t *) i;
-	  if (memcmp (r->signature, ACPI_RSDP_SIG, sizeof (r->signature)) == 0)
-	    {
-	      memcpy (rsdp, r, sizeof (*rsdp));
-	      addr = base + (i - mapped);
-	      break;
-	    }
-	}
-      munmap (mapped, size);
-    }
-  return addr;
-}
-#endif
-
 int
-acpi20_lsapic (char *p)
+acpi_parse_lsapic (acpi_table_entry_header *p)
 {
-  acpi20_entry_lsapic_t *lsapic = (acpi20_entry_lsapic_t *) p;
+  struct acpi_table_lsapic *lsapic = (struct acpi_table_lsapic *) p;
 
-  return ((lsapic->flags & LSAPIC_ENABLED) != 0);
+  return lsapic->flags.enabled;
 }
 
 static int
-acpi20_parse_madt (acpi_madt_t *madt)
+acpi_parse_madt (struct acpi_table_madt *madt)
 {
-  char *p, *end;
+  acpi_table_entry_header *p, *end;
   int n_cpu = 0;
 
-  p = (char *) (madt + 1);
-  end = p + (madt->header.length - sizeof (acpi_madt_t));
+  p = (acpi_table_entry_header *) (madt + 1);
+  end = (acpi_table_entry_header *) ((char *) madt + madt->header.length);
 
   while (p < end)
     {
-      if (*p == ACPI20_ENTRY_LOCAL_SAPIC)
-	n_cpu += acpi20_lsapic (p);
+      if (p->type == ACPI_MADT_LSAPIC)
+	n_cpu += acpi_parse_lsapic (p);
 
-      p += p[1];
+      p = (acpi_table_entry_header *) ((char *) p + p->length);
     }
 
   return n_cpu;
 }
 
 static int
-acpi_lsapic (char *p)
-{
-  acpi_entry_lsapic_t *lsapic = (acpi_entry_lsapic_t *) p;
-
-  if ((lsapic->flags & LSAPIC_PRESENT) == 0
-      || (lsapic->flags & LSAPIC_ENABLED) == 0
-      || lsapic->flags & LSAPIC_PERFORMANCE_RESTRICTED)
-    return 0;
-  return 1;
-}
-
-static int
-acpi_parse_msapic (acpi_sapic_t *msapic)
-{
-  char *p, *end;
-  int n_cpu = 0;
-
-  p = (char *) (msapic + 1);
-  end = p + (msapic->header.length - sizeof (acpi_sapic_t));
-
-  while (p < end)
-    {
-      if (*p == ACPI_ENTRY_LOCAL_SAPIC)
-	n_cpu += acpi_lsapic(p);
-
-      p += p[1];
-    }
-
-  return n_cpu;
-}
-
-static int
-acpi_parse_rsdp (int mem_fd, acpi20_rsdp_t *rsdp)
+acpi_parse_rsdp (int mem_fd, struct acpi20_table_rsdp *rsdp)
 {
   int n_cpu = 0;
   int i;
-  acpi_rsdt_t *rsdt = 0;
-  acpi_xsdt_t *xsdt = 0;
+  struct acpi_table_xsdt *xsdt = 0;
   int tables;
 
-  if (rsdp->xsdt)
-    xsdt = (acpi_xsdt_t *) acpi_map_table (mem_fd, rsdp->xsdt, ACPI_XSDT_SIG);
+  if (rsdp->xsdt_address)
+    xsdt = (struct acpi_table_xsdt *) acpi_map_table (mem_fd, rsdp->xsdt_address,
+						      ACPI_XSDT_SIG);
   if (xsdt)
     {
-      tables = (xsdt->header.length - sizeof (acpi_desc_table_hdr_t)) / 8;
+      tables = (xsdt->header.length - sizeof (struct acpi_table_header)) / 8;
       for (i = 0; i < tables; i++)
 	{
-	  acpi_desc_table_hdr_t *dt
-	    = acpi_map_table (mem_fd, xsdt->entry_ptrs[i], ACPI_MADT_SIG);
+	  struct acpi_table_header *dt
+	    = acpi_map_table (mem_fd, xsdt->entry[i], ACPI_MADT_SIG);
 	  if (dt)
-	    n_cpu += acpi20_parse_madt ((acpi_madt_t *) dt);
+	    n_cpu += acpi_parse_madt ((struct acpi_table_madt *) dt);
 	  acpi_unmap_table (dt);
 	}
-      acpi_unmap_table ((acpi_desc_table_hdr_t *) xsdt);
-    }
-  else
-    {
-      if (rsdp->rsdt)
-	rsdt = (acpi_rsdt_t *) acpi_map_table (mem_fd, rsdp->rsdt,
-					       ACPI_RSDT_SIG);
-      if (rsdt)
-	{
-	  tables = (rsdt->header.length - sizeof (acpi_desc_table_hdr_t)) / 8;
-	  for (i = 0; i < tables; i++)
-	    {
-	      acpi_desc_table_hdr_t *dt
-		= acpi_map_table (mem_fd, rsdt->entry_ptrs[i], ACPI_SAPIC_SIG);
-	      if (dt)
-		n_cpu += acpi_parse_msapic ((acpi_sapic_t *) dt);
-	      acpi_unmap_table (dt);
-	    }
-	  acpi_unmap_table ((acpi_desc_table_hdr_t *) rsdt);
-	}
+      acpi_unmap_table ((struct acpi_table_header *) xsdt);
     }
   return n_cpu;
 }
 
 int ia64DetectSMP(hd_data_t *hd_data)
 {
-  int n_cpu = 0, mem_fd;
-  acpi20_rsdp_t rsdp;
+  int n_cpu = 0, mem_fd, systab_fd;
+  struct acpi20_table_rsdp rsdp;
   uint8_t *mapped;
   unsigned long addr = 0, offset;
   int ok = 0;
   str_list_t *sl;
-  static char *rsd_klog = "ACPI 2.0=";
+  const char *rsd_klog = "ACPI 2.0=";
+  const char *rsd_systab = "ACPI20=";
   char *s;
-#if 0
-  off_t o;
-  int i;
-
-  unsigned long ranges[][2] = {
-    { 0x7fe00000, 0x80000000 - 0x7fe00000 },
-    { 0x3fe00000, 0x40000000 - 0x3fe00000 },
-    { 0x000e0000, 0x00100000 - 0x000e0000 }
-  };
-#endif
 
   mem_fd = open("/dev/mem", O_RDONLY);
   if(mem_fd == -1) return -1;
 
+  systab_fd = open("/proc/efi/systab", O_RDONLY);
+  if (systab_fd != -1)
+    {
+      char buffer[512];
+      int n_read = read(systab_fd, buffer, sizeof(buffer) - 1);
+      close(systab_fd);
+      if (n_read > 0)
+	{
+	  buffer[n_read] = 0;
+	  if ((s = strstr(buffer, rsd_systab)) != NULL &&
+	      sscanf(s + strlen(rsd_systab), "%lx", &addr) == 1)
+	    goto found_it;
+	}
+    }
+
   if(!hd_data->klog) read_klog(hd_data);
 
   for(sl = hd_data->klog; sl; sl = sl->next) {
     if((s = strstr(sl->str, rsd_klog))) {
       if(sscanf(s + strlen(rsd_klog), "%lx", &addr) == 1) {
-//        addr &= ~(0xfL << 60);
+      found_it:
 	offset= PAGE_OFFSET (addr);
         mapped = mmap(NULL, sizeof rsdp + offset, PROT_READ, MAP_PRIVATE,
 		      mem_fd, (unsigned long) addr - offset);
@@ -857,23 +750,6 @@ int ia64DetectSMP(hd_data_t *hd_data)
     }
   }
 
-#if 0
-  for(i = 0; i < sizeof ranges / sizeof *ranges; i++) {
-    if(ranges[i][0]) {
-      ADD2LOG("looking for RSDP in 0x%lx - 0x%lx\n",
-        ranges[i][0], ranges[i][0] + ranges[i][1] - 1
-      );
-      if((addr = acpi_find_rsdp(mem_fd, ranges[i][0], ranges[i][1], &rsdp))) {
-        n_cpu = acpi_parse_rsdp(mem_fd, &rsdp);
-        if(n_cpu) {
-          ADD2LOG("RSDP found at 0x%lx\n", addr);
-          break;
-        }
-      }
-    }
-  }
-#endif
-
   if(ok) {
     n_cpu = acpi_parse_rsdp(mem_fd, &rsdp);
     if(n_cpu) {
@@ -891,26 +767,3 @@ int ia64DetectSMP(hd_data_t *hd_data)
 
 #endif	/* __ia64__ */
 
-#else	/* 0 */
-
-int ia64DetectSMP(hd_data_t *hd_data)
-{
-  str_list_t *sl;
-  unsigned u1 = 0, u2;
-
-  if(!hd_data->klog) read_klog(hd_data);
-
-  for(sl = hd_data->klog; sl; sl = sl->next) {
-    if(sscanf(sl->str, "<4> %u CPUs available, %u CPUs total", &u1, &u2) == 2) {
-      ADD2LOG("cpus: %u available, %u total\n", u1, u2);
-      break;
-    }
-  }
-
-  return u1;
-}
-
-
-
-#endif	/* 0 */
-
