@@ -13,7 +13,7 @@ extern "C" {
  * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  */
 
-#define HD_VERSION	7
+#define HD_VERSION	8
 
 /*
  * debug flags
@@ -94,19 +94,18 @@ extern "C" {
  * flags to control the probing.
  */
 typedef enum probe_feature {
-  pr_memory = 1, pr_pci, pr_pci_range, pr_pci_ext, pr_isapnp, pr_cdrom,
-  pr_cdrom_info, pr_net, pr_floppy, pr_misc, pr_misc_serial, pr_misc_par,
-  pr_misc_floppy, pr_serial, pr_cpu, pr_bios, pr_monitor, pr_mouse, pr_ide,
-  pr_scsi, pr_scsi_geo, pr_scsi_cache, pr_usb, pr_usb_mods, pr_adb,
-  pr_modem, pr_modem_usb, pr_parallel, pr_parallel_lp, pr_parallel_zip,
-  pr_isa, pr_isa_isdn, pr_dac960, pr_smart, pr_isdn, pr_kbd, pr_prom,
-  pr_sbus, pr_int, pr_braille, pr_braille_alva, pr_braille_fhp,
-  pr_braille_ht, pr_ignx11, pr_sys, pr_dasd, pr_i2o, pr_cciss, pr_bios_vbe,
-  pr_isapnp_old, pr_isapnp_new, pr_isapnp_mod, pr_braille_baum, pr_manual,
-  pr_fb, pr_bios_vbe2, pr_veth, pr_partition, pr_disk, pr_ataraid, pr_pppoe,
-  pr_scan, pr_partition_add, pr_pcmcia, pr_fork, pr_parallel_imm, pr_s390,
-  /* pr_bios_32, */
-  pr_max, pr_lxrc, pr_default, pr_all		/* pr_all must be last */
+  pr_memory = 1, pr_pci, pr_isapnp, pr_net, pr_floppy, pr_misc,
+  pr_misc_serial, pr_misc_par, pr_misc_floppy, pr_serial, pr_cpu, pr_bios,
+  pr_monitor, pr_mouse, pr_scsi, pr_usb, pr_usb_mods, pr_adb, pr_modem,
+  pr_modem_usb, pr_parallel, pr_parallel_lp, pr_parallel_zip, pr_isa,
+  pr_isa_isdn, pr_isdn, pr_kbd, pr_prom, pr_sbus, pr_int, pr_braille,
+  pr_braille_alva, pr_braille_fhp, pr_braille_ht, pr_ignx11, pr_sys,
+  pr_bios_vbe, pr_isapnp_old, pr_isapnp_new, pr_isapnp_mod, pr_braille_baum,
+  pr_manual, pr_fb, pr_veth, pr_pppoe, pr_scan, pr_pcmcia, pr_fork,
+  pr_parallel_imm, pr_s390, pr_cpuemu, pr_sysfs, pr_s390disks, pr_udev,
+  pr_block, pr_block_cdrom, pr_block_part, pr_edd, pr_edd_mod, pr_bios_ddc,
+  pr_bios_fb, pr_bios_mode, pr_input, pr_block_mods, pr_bios_vesa,
+  pr_max, pr_lxrc, pr_dsl, pr_default, pr_all		/* pr_all must be last */
 } hd_probe_feature_t;
 
 /*
@@ -124,7 +123,7 @@ typedef enum hw_item {
   hw_manual, hw_usb_ctrl, hw_usb, hw_bios, hw_pci, hw_isapnp, hw_bridge,
   hw_hub, hw_scsi, hw_ide, hw_memory, hw_dvb, hw_pcmcia, hw_pcmcia_ctrl,
   hw_ieee1394, hw_ieee1394_ctrl, hw_hotplug, hw_hotplug_ctrl, hw_zip, hw_pppoe,
-  hw_wlan,	/* append new entries here */
+  hw_wlan, hw_redasd, hw_dsl, hw_block, hw_tape, hw_vbe, /* append new entries here */
   hw_unknown, hw_all					/* hw_all must be last */
 } hd_hw_item_t;
 
@@ -144,7 +143,7 @@ typedef enum base_classes {
   bc_monitor = 0x100, bc_internal, bc_modem, bc_isdn, bc_ps2, bc_mouse,
   bc_storage_device, bc_network_interface, bc_keyboard, bc_printer,
   bc_hub, bc_braille, bc_scanner, bc_joystick, bc_chipcard, bc_camera,
-  bc_framebuffer, bc_dvb, bc_tv, bc_partition
+  bc_framebuffer, bc_dvb, bc_tv, bc_partition, bc_dsl
 } hd_base_classes_t;
 
 /* subclass values of bc_monitor */
@@ -243,9 +242,14 @@ typedef enum sc_camera {
 
 /* subclass values of bc_modem */
 typedef enum sc_modem {
-  sc_mod_at, sc_mod_win1, sc_mod_win2, sc_mod_win3
+  sc_mod_at, sc_mod_win1, sc_mod_win2, sc_mod_win3, sc_mod_win4
 } hd_sc_modem_t;
 
+/* subclass values of bc_dsl */
+typedef enum sc_dsl {
+  sc_dsl_unknown, sc_dsl_pppoe, sc_dsl_capi, sc_dsl_capiisdn
+} hd_sc_dsl_t;
+
 /* prog_if's of sc_ser_usb */
 typedef enum pif_usb_e {
   pif_usb_uhci = 0, pif_usb_ohci = 0x10, pif_usb_ehci = 0x20,
@@ -257,6 +261,11 @@ typedef enum pif_cdrom {
   pif_cdrom, pif_cdr, pif_cdrw, pif_dvd, pif_dvdr, pif_dvdram
 } hd_pif_cdrom_t ;
 
+/* S/390 disk prog_if values */
+typedef enum pif_s390disk {
+  pif_scsi, pif_dasd
+} hd_pif_s390disk_t;
+
 /* bus type values similar to PCI bridge subclasses */
 typedef enum bus_types {
   bus_none, bus_isa, bus_eisa, bus_mc, bus_pci, bus_pcmcia, bus_nubus,
@@ -368,6 +377,14 @@ typedef struct s_str_list_t {
 } str_list_t;
 
 
+typedef struct {
+  unsigned char bitmap[16];	/* large enough for all uses */
+  unsigned bits;		/* real bitmap length in bits */
+  unsigned not_empty:1;		/* at least 1 bit is set */
+  str_list_t *str;		/* interpreted bitmask */
+} hd_bitmap_t;
+
+
 /*
  * for memory areas
  */
@@ -463,19 +480,23 @@ typedef enum {
   sm_memdevicemap, sm_mouse, sm_battery, sm_reset,
   sm_secure, sm_power, sm_voltage, sm_cool,
   sm_temperature, sm_current, sm_outofband, sm_bis,
-  sm_boot, sm_memerror64, sm_mandev, sm_mandevcomp,
+  sm_boot, sm_mem64error, sm_mandev, sm_mandevcomp,
   sm_mdtd, sm_inactive = 126, sm_end = 127
 } hd_smbios_type_t;
 
+
+/* common part of all smbios_* types */
 typedef struct {
-  union u_hd_smbios_t *next;
-  hd_smbios_type_t type;
-  int data_len;
-  unsigned char *data;
-  str_list_t *strings;
-  int handle;
+  union u_hd_smbios_t *next;	/* link to next entry */
+  hd_smbios_type_t type;	/* BIOS info type */
+  int data_len;			/* formatted section length */
+  unsigned char *data;		/* formatted section */
+  str_list_t *strings;		/* strings taken from the unformed section */
+  int handle;			/* handle, unique 16 bit number */
 } smbios_any_t;
 
+
+/* BIOS related information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -483,13 +504,16 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *vendor;
-  char *version;
-  char *date;
-  uint64_t features;
-  unsigned xfeatures;
+  char *vendor;			/* vendor name */
+  char *version;		/* version (free form) */
+  char *date;			/* date mm/dd/yyyy (old: yy) */
+  hd_bitmap_t feature;		/* BIOS characteristics */
+  unsigned start;		/* start address */
+  unsigned rom_size;		/* ROM size (in bytes) */
 } smbios_biosinfo_t;
 
+
+/* overall system related information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -497,13 +521,16 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *manuf;
-  char *product;
-  char *version;
-  char *serial;
-  unsigned wake_up;
+  char *manuf;			/* manufacturer */
+  char *product;		/* product name */
+  char *version;		/* version */
+  char *serial;			/* serial number */
+  unsigned char uuid[16];	/* universal unique id; all 0x00: undef, all 0xff: undef but settable */
+  hd_id_t wake_up;		/* wake-up type */
 } smbios_sysinfo_t;
 
+
+/* motherboard related information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -511,12 +538,21 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *manuf;
-  char *product;
-  char *version;
-  char *serial;
+  char *manuf;			/* manufacturer */
+  char *product;		/* product name */
+  char *version;		/* version */
+  char *serial;			/* serial number */
+  char *asset;			/* asset tag */
+  hd_id_t board_type;		/* board type */
+  hd_bitmap_t feature;		/* board features */
+  char *location;		/* location in chassis */
+  int chassis;			/* handle of chassis */
+  int objects_len;		/* number of contained objects */
+  int *objects;			/* array of object handles */
 } smbios_boardinfo_t;
 
+
+/* chassis information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -524,10 +560,21 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *manuf;
-  unsigned ch_type;
+  char *manuf;			/* manufacturer */
+  char *version;		/* version */
+  char *serial;			/* serial number */
+  char *asset;			/* asset tag */
+  hd_id_t ch_type;		/* chassis type */
+  unsigned lock;		/* 1: lock present, 0: not present or unknown */
+  hd_id_t bootup;		/* bootup state */
+  hd_id_t power;		/* power supply state (at last boot) */
+  hd_id_t thermal;		/* thermal state (at last boot) */
+  hd_id_t security;		/* security state (at last boot) */
+  unsigned oem;			/* OEM-specific information */
 } smbios_chassis_t;
 
+
+/* processor information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -535,17 +582,29 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *socket;
-  char *manuf;
-  char *version;
+  char *socket;			/* socket */
+  hd_id_t upgrade;		/* socket type */
+  char *manuf;			/* manufacturer */
+  char *version;		/* version */
+  char *serial;			/* serial number */
+  char *asset;			/* asset tag */
+  char *part;			/* part number */
+  hd_id_t pr_type;		/* processor type */
+  hd_id_t family;		/* processor family */
+  uint64_t cpu_id;		/* processor id */
   unsigned voltage;		/* in 0.1 V */
   unsigned ext_clock;		/* MHz */
   unsigned max_speed;		/* MHz */
   unsigned current_speed;	/* MHz */
-  unsigned status;
-  unsigned upgrade;
+  unsigned sock_status;		/* socket status (1: populated, 0: empty */
+  hd_id_t cpu_status;		/* cpu status */
+  int l1_cache;			/* handle of L1 cache */
+  int l2_cache;			/* handle of L2 cache */
+  int l3_cache;			/* handle of L3 cache */
 } smbios_processor_t;
 
+
+/* cache information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -553,10 +612,75 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *descr[8];
-  unsigned dtype[8];
+  char *socket;			/* socket designation */
+  unsigned max_size;		/* max cache size in kbytes */
+  unsigned current_size;	/* current size in kbytes */
+  unsigned speed;		/* cache speed in nanoseconds */
+  hd_id_t mode;			/* operational mode */
+  unsigned state;		/* 0/1: disabled/enabled */
+  hd_id_t location;		/* cache location */
+  unsigned socketed;		/* 0/1: not socketed/socketed */
+  unsigned level;		/* cache level (0 = L1, 1 = L2, ...) */
+  hd_id_t ecc;			/* error correction type */
+  hd_id_t cache_type;		/* logical cache type */
+  hd_id_t assoc;		/* cache associativity */
+  hd_bitmap_t supp_sram;	/* supported SRAM types */
+  hd_bitmap_t sram;		/* current SRAM type */
+} smbios_cache_t;
+
+
+/* port connector information */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  hd_id_t port_type;		/* port type */
+  char *i_des;			/* internal reference designator */
+  hd_id_t i_type;		/* internal connector type */
+  char *x_des;			/* external reference designator */
+  hd_id_t x_type;		/* external connector type */
+} smbios_connect_t;
+
+
+/* system slot information */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  char *desig;			/* slot designation */
+  hd_id_t slot_type;		/* slot type */
+  hd_id_t bus_width;		/* data bus width */
+  hd_id_t usage;		/* current usage */
+  hd_id_t length;		/* slot length */
+  unsigned id;			/* slot id */
+  hd_bitmap_t feature;		/* slot characteristics */
+} smbios_slot_t;
+
+
+/* on board devices information */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  unsigned dev_len;		/* device list length */
+  struct {
+    char *name;			/* device name */
+    hd_id_t type;		/* device type */
+    unsigned status;		/* 0: disabled, 1: enabled */
+  } *dev;			/* device list  */
 } smbios_onboard_t;
 
+
+/* OEM information */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -564,9 +688,35 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *current;
+  str_list_t *oem_strings;	/* OEM strings */
+} smbios_oem_t;
+
+
+/* system config options */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  str_list_t *options;		/* system config options */
+} smbios_config_t;
+
+
+/* language information */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;		/* list of languages */
+  int handle;
+  char *current;		/* current language */
 } smbios_lang_t;
 
+
+/* group associations */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -574,10 +724,30 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  unsigned ecc;
-  unsigned max_size;		/* kB */
+  char *name;			/* group name */
+  int items_len;		/* number of items in this group */
+  int *item_handles;		/* array of item handles */
+} smbios_group_t;
+
+
+/* physical memory array (consists of several memory devices) */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  hd_id_t location;		/* memory device location */
+  hd_id_t use;			/* memory usage */
+  hd_id_t ecc;			/* ECC types */
+  unsigned max_size;		/* maximum memory size in kB */
+  int error_handle;		/* points to error info record; 0xfffe: not supported, 0xffff: no error */
+  unsigned slots;		/* slots or sockets for this device */
 } smbios_memarray_t;
 
+
+/* memory device */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -585,17 +755,26 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  char *location;
-  char *bank;
-  unsigned width;
-  unsigned eccbits;
+  char *location;		/* device location */
+  char *bank;			/* bank location */
+  char *manuf;			/* manufacturer */
+  char *serial;			/* serial number */
+  char *asset;			/* asset tag */
+  char *part;			/* part number */
+  int array_handle;		/* memory array this device belongs to */
+  int error_handle;		/* points to error info record; 0xfffe: not supported, 0xffff: no error */
+  unsigned width;		/* data width in bits */
+  unsigned eccbits;		/* ecc bits */
   unsigned size;		/* kB */
-  unsigned form;
-  unsigned type1;
-  unsigned type2;
-  unsigned speed;
+  hd_id_t form;			/* form factor */
+  unsigned set;			/* 0: does not belong to a set; 1-0xfe: set number; 0xff: unknown */
+  hd_id_t mem_type;		/* memory type */
+  hd_bitmap_t type_detail;	/* memory type details */
+  unsigned speed;		/* in MHz */
 } smbios_memdevice_t;
 
+
+/* 32-bit memory error information  */
 typedef struct {
   union u_hd_smbios_t *next;
   hd_smbios_type_t type;
@@ -603,11 +782,112 @@ typedef struct {
   unsigned char *data;
   str_list_t *strings;
   int handle;
-  unsigned mtype;
-  unsigned interface;
-  unsigned buttons;
+  hd_id_t err_type;		/* error type memory */
+  hd_id_t granularity;		/* memory array or memory partition */
+  hd_id_t operation;		/* mem operation causing the error */
+  unsigned syndrome;		/* vendor-specific ECC syndrome; 0: unknown */
+  unsigned array_addr;		/* fault address rel. to mem array; 0x80000000: unknown */
+  unsigned device_addr;		/* fault address rel to mem device; 0x80000000: unknown */
+  unsigned range;		/* range, within which the error can be determined; 0x80000000: unknown */
+} smbios_memerror_t;
+
+
+/* memory array mapped address */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  int array_handle;		/* memory array this mapping belongs to */
+  uint64_t start_addr;		/* memory range start address */
+  uint64_t end_addr;		/* end address */
+  unsigned part_width;		/* number of memory devices */
+} smbios_memarraymap_t;
+
+
+/* memory device mapped address */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  int memdevice_handle;		/* memory device handle */
+  int arraymap_handle;		/* memory array mapping handle */
+  uint64_t start_addr;		/* memory range start address */
+  uint64_t end_addr;		/* end address */
+  unsigned row_pos;		/* position of the referenced memory device in a row of the address partition */
+  unsigned interleave_pos;	/* dto, in an interleave */
+  unsigned interleave_depth;	/* number of consecutive rows */
+} smbios_memdevicemap_t;
+
+
+/* pointing device (aka 'mouse') information */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  hd_id_t mtype;		/* mouse type */
+  hd_id_t interface;		/* interface type */
+  unsigned buttons;		/* number of buttons */
 } smbios_mouse_t;
 
+
+/* hardware security */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  hd_id_t power;		/* power-on password status */
+  hd_id_t keyboard;		/* keyboard password status */
+  hd_id_t admin;		/* admin password status */
+  hd_id_t reset;		/* front panel reset status */
+} smbios_secure_t;
+
+
+/* system power controls */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  unsigned month;		/* next scheduled power-on month */
+  unsigned day;			/* dto, day */
+  unsigned hour;		/* dto, hour */
+  unsigned minute;		/* dto, minute */
+  unsigned second;		/* dto, second */
+} smbios_power_t;
+
+
+/* 64-bit memory error information  */
+typedef struct {
+  union u_hd_smbios_t *next;
+  hd_smbios_type_t type;
+  int data_len;
+  unsigned char *data;
+  str_list_t *strings;
+  int handle;
+  hd_id_t err_type;		/* error type memory */
+  hd_id_t granularity;		/* memory array or memory partition */
+  hd_id_t operation;		/* mem operation causing the error */
+  unsigned syndrome;		/* vendor-specific ECC syndrome; 0: unknown */
+  uint64_t array_addr;		/* fault address rel. to mem array; 0x80000000: unknown */
+  uint64_t device_addr;		/* fault address rel to mem device; 0x80000000: unknown */
+  unsigned range;		/* range, within which the error can be determined; 0x80000000: unknown */
+} smbios_mem64error_t;
+
+
 typedef union u_hd_smbios_t {
   union u_hd_smbios_t *next;  
   smbios_any_t any;
@@ -616,14 +896,58 @@ typedef union u_hd_smbios_t {
   smbios_boardinfo_t boardinfo;
   smbios_chassis_t chassis;
   smbios_processor_t processor;
+  smbios_cache_t cache;
+  smbios_connect_t connect;
+  smbios_slot_t slot;
+  smbios_onboard_t onboard;
+  smbios_oem_t oem;
+  smbios_config_t config;
   smbios_lang_t lang;
+  smbios_group_t group;
   smbios_memarray_t memarray;
   smbios_memdevice_t memdevice;
+  smbios_memerror_t memerror;
+  smbios_memarraymap_t memarraymap;
+  smbios_memdevicemap_t memdevicemap;
   smbios_mouse_t mouse;
-  smbios_onboard_t onboard;
+  smbios_secure_t secure;
+  smbios_power_t power;
+  smbios_mem64error_t mem64error;
 } hd_smbios_t;
 
 
+/*
+ * udev database info
+ */
+typedef struct s_udevinfo_t {
+  struct s_udevinfo_t *next;
+  char *sysfs;
+  char *name;
+  str_list_t *links;
+} hd_udevinfo_t;
+
+
+/*
+ * sysfs driver info
+ */
+typedef struct s_sysfsdrv_t {
+  struct s_sysfsdrv_t *next;
+  char *driver;
+  char *device;
+} hd_sysfsdrv_t;
+
+
+/*
+ * device number; type is either 0 or 'b' or 'c'.
+ *
+ * range: number of nodes
+ */
+typedef struct {
+  int type;
+  unsigned major, minor, range;
+} hd_dev_num_t;
+
+
 /*
  * structure holding the (raw) PCI data
  */
@@ -642,10 +966,13 @@ typedef struct s_pci_t {
   unsigned base_class, sub_class, prog_if;	/* PCI device classes */
   unsigned dev, vend, sub_dev, sub_vend, rev;	/* vendor & device ids */
   unsigned irq;					/* used irq, if any */
-  uint64_t base_addr[6];			/* I/O or memory base */
-  uint64_t base_len[6];				/* I/O or memory ranges */
+  uint64_t base_addr[7];			/* I/O or memory base */
+  uint64_t base_len[7];				/* I/O or memory ranges */
+  unsigned addr_flags[7];			/* I/O or memory address flags */
   uint64_t rom_base_addr;			/* memory base for card ROM */
   uint64_t rom_base_len;			/* memory range for card ROM */
+  char *sysfs_id;				/* sysfs path */
+  char *sysfs_bus_id;				/* sysfs bus id */
 } pci_t;
 
 /*
@@ -747,6 +1074,9 @@ typedef struct scsi_s {
   unsigned pci_bus;
   unsigned pci_slot;
   unsigned pci_func;
+  uint64_t wwpn;
+  uint64_t fcp_lun;
+  char *controller_id;
 } scsi_t;
 
 
@@ -766,6 +1096,14 @@ typedef struct devtree_s {
   unsigned char *edid;                  /* 128 bytes */
 } devtree_t;
 
+/*
+ * Device/CU model numbers for S/390
+ */
+typedef struct ccw_s {
+  unsigned char lcss;
+  unsigned char cu_model;
+  unsigned char dev_model;
+} ccw_t;
 
 /*
  * special CDROM entry
@@ -977,7 +1315,7 @@ typedef struct {
 typedef enum resource_types {
   res_any, res_phys_mem, res_mem, res_io, res_irq, res_dma, res_monitor,
   res_size, res_disk_geo, res_cache, res_baud, res_init_strings, res_pppd_option,
-  res_framebuffer
+  res_framebuffer, res_hwaddr
 } hd_resource_types_t;
 
 
@@ -1108,6 +1446,12 @@ typedef struct {
   unsigned mode;			/* mode number for kernel */
 } res_framebuffer_t;
 
+typedef struct {
+  union u_hd_res_t *next;
+  enum resource_types type;
+  char *addr;
+} res_hwaddr_t;
+
 typedef union u_hd_res_t {
   union u_hd_res_t *next;  
   res_any_t any;
@@ -1124,6 +1468,7 @@ typedef union u_hd_res_t {
   res_init_strings_t init_strings;
   res_pppd_option_t pppd_option;
   res_framebuffer_t framebuffer;
+  res_hwaddr_t hwaddr;
 } hd_res_t;
 
 
@@ -1205,7 +1550,7 @@ typedef struct isdn_parm_s {
 
 /* device driver info types */
 typedef enum driver_info_type {
-  di_any, di_display, di_module, di_mouse, di_x11, di_isdn, di_kbd
+  di_any, di_display, di_module, di_mouse, di_x11, di_isdn, di_kbd, di_dsl
 } hd_driver_info_t;
 
 /* unspecific info */
@@ -1278,6 +1623,15 @@ typedef struct {
   isdn_parm_t *params;			/* isdn parameters */
 } driver_info_isdn_t;
 
+/* dsl info */
+typedef struct {
+  union driver_info_u *next;
+  enum driver_info_type type;		/* driver info type */
+  str_list_t *hddb0, *hddb1;		/* the actual driver database entries */
+  char *mode;				/* DSL driver types */
+  char *name;				/* DSL driver name */
+} driver_info_dsl_t;
+
 /* keyboard info */
 typedef struct {
   union driver_info_u *next;
@@ -1300,6 +1654,7 @@ typedef union driver_info_u {
   driver_info_x11_t x11;
   driver_info_display_t display;
   driver_info_isdn_t isdn;
+  driver_info_dsl_t dsl;
   driver_info_kbd_t kbd;
 } driver_info_t;
 
@@ -1312,7 +1667,8 @@ typedef union driver_info_u {
 typedef enum hd_detail_type {
   hd_detail_pci, hd_detail_usb, hd_detail_isapnp, hd_detail_cdrom,
   hd_detail_floppy, hd_detail_bios, hd_detail_cpu, hd_detail_prom,
-  hd_detail_monitor, hd_detail_sys, hd_detail_scsi, hd_detail_devtree
+  hd_detail_monitor, hd_detail_sys, hd_detail_scsi, hd_detail_devtree,
+  hd_detail_ccw
 } hd_detail_type_t;
 
 typedef struct {
@@ -1375,6 +1731,11 @@ typedef struct {
   devtree_t *data;
 } hd_detail_devtree_t;
 
+typedef struct {
+  enum hd_detail_type type;
+  ccw_t *data;
+} hd_detail_ccw_t;
+
 typedef union {
   enum hd_detail_type type;
   hd_detail_pci_t pci;
@@ -1389,6 +1750,7 @@ typedef union {
   hd_detail_sys_t sys;
   hd_detail_scsi_t scsi;
   hd_detail_devtree_t devtree;
+  hd_detail_ccw_t ccw;
 } hd_detail_t;
 
 
@@ -1550,21 +1912,54 @@ typedef struct s_hd_t {
    */
   unsigned attached_to;
 
+  /**
+   * sysfs entry for this hardware, if any.
+   */
+  char *sysfs_id;
+
+  /**
+   * sysfs bus id for this hardware, if any.
+   */
+  char *sysfs_bus_id;
+
+  /**
+   * sysfs device link.
+   */
+  char *sysfs_device_link;
+
   /**
    * Special %device file.
-   * Device file name to acces this hardware. Normally something below /dev.
+   * Device file name to access this hardware. Normally something below /dev.
    * For network interfaces this is the interface name.
    */
   char *unix_dev_name;
 
+  /**
+   * Device type & number according to sysfs.
+   */
+  hd_dev_num_t unix_dev_num;
+
+  /**
+   * List of %device names.
+   * Device file names to access this hardware. Normally something below /dev.
+   * They should be all equivalent. The preferred name however is
+   * \ref hd_t::unix_dev_name.
+   */
+  str_list_t *unix_dev_names;
+
   /**
    * Special %device file.
-   * Device file name to acces this hardware. Most hardware only has one
+   * Device file name to access this hardware. Most hardware only has one
    * %device name stored in \ref hd_t::unix_dev_name. But in some cases
    * there's an alternative name.
    */
   char *unix_dev_name2;
 
+  /**
+   * Device type & number according to sysfs.
+   */
+  hd_dev_num_t unix_dev_num2;
+
   /**
    * BIOS/PROM id.
    * Where appropriate, this is a special BIOS/PROM id (e.g. "0x80" for
@@ -1585,6 +1980,9 @@ typedef struct s_hd_t {
    */
   char *unique_id;
 
+  /* List of ids. */
+  str_list_t *unique_ids;
+
   /**
    * (Internal) Probing module that created this entry.
    */
@@ -1681,6 +2079,11 @@ typedef struct s_hd_t {
    */
   char *driver;
 
+  /**
+   * List of currently active drivers.
+   */
+  str_list_t *drivers;
+
   /**
    * Old \ref unique_id for compatibility.
    * The calculation of unique ids has changed in libhd v3.17. Basically
@@ -1778,14 +2181,22 @@ typedef struct {
     unsigned fast:1;		/**< Don't check tricky hardware. */
     unsigned list_md:1;		/**< Report md & lvm devices from /proc/partitions */
     unsigned nofork:1;		/**< don't run potentially hanging code in a subprocess */
+    unsigned nosysfs:1;		/**< don't ask sysfs */
     unsigned forked:1;		/**< we're running in a subprocess */
+    unsigned cpuemu:1;		/**< use CPU emulation to run BIOS code (i386 only) */
+    unsigned udev:1;		/**< return first udev symlink as device name */
+    unsigned edd_used:1;	/**< internal: edd info has been used  */
   } flags;
 
 
+  /** Concentrate on tses devices.
+   * List of sysfs ids for devices to look for.
+   */
+  str_list_t *only;
+
   /*
    * The following entries should *not* be accessed outside of libhd!
    */
-
   unsigned char probe[(pr_all + 7) / 8];	/**< (Internal) bitmask of probing features. */
   unsigned char probe_set[(pr_all + 7) / 8];	/**< (Iternal) bitmask of probing features that will always be set. */
   unsigned char probe_clr[(pr_all + 7) / 8];	/**< (Internal) bitmask of probing features that will always be reset. */
@@ -1807,6 +2218,7 @@ typedef struct {
   str_list_t *klog;		/**< (Internal) kernel log */
   str_list_t *proc_usb;		/**< (Internal) /proc/bus/usb info */
   usb_t *usb;			/**< (Internal) usb info */
+  hddb_pci_t *hddb_pci_hm;	/**< (Internal) pci module info */
   hddb_pci_t *hddb_pci;		/**< (Internal) pci module info */
   hddb2_data_t *hddb2[2];	/**< (Internal) hardware database */
   str_list_t *kmods;		/**< (Internal) list of active kernel modules */
@@ -1825,6 +2237,7 @@ typedef struct {
   hd_manual_t *manual;		/**< (Internal) hardware config info */
   str_list_t *disks;		/**< (Internal) disks according to /proc/partitions */
   str_list_t *partitions;	/**< (Internal) dto, partitions */
+  str_list_t *cdroms;		/**< (Internal) cdroms according to PROC_CDROM_INFO */
   hd_smbios_t *smbios;		/**< (Internal) smbios data */
   struct {
     unsigned ok:1;
@@ -1834,6 +2247,11 @@ typedef struct {
     int id;
     int updated;
   } shm;			/**< (Internal) our shm segment */
+  unsigned pci_config_type;	/**< (Internal) PCI config type (1 or 2), 0: unknown */
+  hd_udevinfo_t *udevinfo;	/**< (Internal) udev info */
+  hd_sysfsdrv_t *sysfsdrv;	/**< (Internal) sysfs driver info */
+  uint64_t sysfsdrv_id;		/**< (Internal) sysfs driver info id */
+  str_list_t *scanner_db;	/**< (Internal) list of scanner modules */
 } hd_data_t;
 
 
@@ -1880,6 +2298,7 @@ int hd_usb_support(hd_data_t *hd_data);
 int hd_smp_support(hd_data_t *hd_data);
 int hd_mac_color(hd_data_t *hd_data);
 int hd_color(hd_data_t *hd_data);
+int hd_is_uml(hd_data_t *hd_data);
 unsigned hd_display_adapter(hd_data_t *hd_data);
 unsigned hd_boot_disk(hd_data_t *hd_data, int *matches);
 enum cpu_arch hd_cpu_arch(hd_data_t *hd_data);
@@ -1890,9 +2309,12 @@ hd_t *hd_get_device_by_idx(hd_data_t *hd_data, unsigned idx);
 void hd_set_hw_class(hd_t *hd, hd_hw_item_t hw_class);
 int hd_is_hw_class(hd_t *hd, hd_hw_item_t hw_class);
 
+char *hd_version(void);
+
 /* implemented in hddb.c */
 
 str_list_t *get_hddb_packages(hd_data_t *hd_data);
+void hddb_add_info(hd_data_t *hd_data, hd_t *hd);
 
 void hddb_dump_raw(hddb2_data_t *hddb, FILE *f);
 void hddb_dump(hddb2_data_t *hddb, FILE *f);
@@ -1924,7 +2346,7 @@ int hd_change_status(const char *id, hd_status_t status, const char *config_stri
 
 /* (C) 2003 kkeil@suse.de */
 
-#define CDBISDN_VERSION	0x0100
+#define CDBISDN_VERSION	0x0101
 
 #ifndef PCI_ANY_ID
 #define PCI_ANY_ID	0xffff
@@ -1946,10 +2368,10 @@ typedef struct {
 typedef struct	{
 	int	handle;		/* internal identifier idx in database */
 	int	vhandle;	/* internal identifier to vendor database */
-	const char *name;	/* cardname */
-	const char *lname;	/* vendor short name + cardname */
-	const char *Class;	/* CLASS of the card */
-	const char *bus;	/* bus type */
+	char	*name;		/* cardname */
+	char	*lname;		/* vendor short name + cardname */
+	char	*Class;		/* CLASS of the card */
+	char	*bus;		/* bus type */
 	int	revision;	/* revision used with USB */
 	int	vendor;		/* Vendor ID for ISAPNP and PCI cards */
 	int     device;		/* Device ID for ISAPNP and PCI cards */
@@ -1970,22 +2392,22 @@ typedef struct  {
 	int	typ;		/* Type to identify the driver */
 	int	subtyp;		/* Subtype of the driver type */
 	int	smp;		/* SMP supported ? */
-	const char *mod_name;	/* name of the driver module */
-	const char *para_str;	/* optional parameter string */
-	const char *mod_preload;/* optional modules to preload */
-	const char *cfg_prog;	/* optional cfg prog */
-	const char *firmware;	/* optional firmware to load */
-	const char *description;/* optional description */
-	const char *need_pkg;	/* list of packages needed for function */
-	const char *info;	/* optional additional info */
-	const char *protocol;	/* supported D-channel protocols */
-	const char *interface;  /* supported API interfaces */
-	const char *io;		/* possible IO ports with legacy ISA cards */
-	const char *irq;	/* possible interrupts with legacy ISA cards */
-	const char *membase;	/* possible membase with legacy ISA cards */
-	const char *features;	/* optional features*/
-	int card_ref;		/* reference to a card */
-	const char *name;	/* driver name */
+	char 	*mod_name;	/* name of the driver module */
+	char 	*para_str;	/* optional parameter string */
+	char 	*mod_preload;	/* optional modules to preload */
+	char 	*cfg_prog;	/* optional cfg prog */
+	char 	*firmware;	/* optional firmware to load */
+	char 	*description;	/* optional description */
+	char 	*need_pkg;	/* list of packages needed for function */
+	char 	*info;		/* optional additional info */
+	char 	*protocol;	/* supported D-channel protocols */
+	char 	*interface;	/* supported API interfaces */
+	char 	*io;		/* possible IO ports with legacy ISA cards */
+	char 	*irq;		/* possible interrupts with legacy ISA cards */
+	char 	*membase;	/* possible membase with legacy ISA cards */
+	char 	*features;	/* optional features*/
+	int 	card_ref;	/* reference to a card */
+	char 	*name;		/* driver name */
 } cdb_isdn_vario;
 
 
