@@ -1,3 +1,6 @@
+#include <sysfs/dlist.h>
+#include <sysfs/libsysfs.h>
+
 #define PROC_CMDLINE		"/proc/cmdline"
 #define PROC_PCI_DEVICES	"/proc/bus/pci/devices"
 #define PROC_PCI_BUS		"/proc/bus/pci"
@@ -9,7 +12,6 @@
 #define PROC_NVRAM_24		"/proc/nvram"
 #define PROC_IDE		"/proc/ide"
 #define PROC_SCSI		"/proc/scsi"
-#define PROC_SCSI_SCSI		"/proc/scsi/scsi"
 #define PROC_CDROM_INFO		"/proc/sys/dev/cdrom/info"
 #define PROC_NET_IF_INFO	"/proc/net/dev"
 #define PROC_MODULES		"/proc/modules"
@@ -18,14 +20,10 @@
 #define PROC_PARPORT_22		"/proc/parport/"			/* Final '/' is essential! */
 #define PROC_PARPORT_24		"/proc/sys/dev/parport/parport"
 #define PROC_KCORE		"/proc/kcore"
-#define PROC_USB_DEVICES	"/proc/bus/usb/devices"
-#define PROC_DAC960		"/proc/rd"
-#define PROC_SMART_22		"/proc/array"
-#define PROC_SMART_24		"/proc/driver/array"
-#define PROC_SMART_24_NEW	"/proc/driver/cpqarray"
+// #define PROC_USB_DEVICES	"/proc/bus/usb/devices"
+#define PROC_USB_DEVICES	"/proc/bus/usb/devices_please-use-sysfs-instead"
 #define PROC_PROM		"/proc/device-tree"
 #define PROC_MEMINFO		"/proc/meminfo"
-#define PROC_DASD		"/proc/dasd"
 #define PROC_VERSION		"/proc/version"
 #define PROC_ISAPNP		"/proc/isapnp"
 #define PROC_ISERIES		"/proc/iSeries"
@@ -37,21 +35,17 @@
 #define DEV_PSAUX		"/dev/psaux"
 #define DEV_ADBMOUSE		"/dev/adbmouse"
 #define DEV_MEM			"/dev/mem"
-#define DEV_DAC960		"/dev/rd"
-#define DEV_SMART		"/dev/ida"
 #define DEV_KBD			"/dev/kbd"
 #define DEV_CONSOLE		"/dev/console"
 #define DEV_OPENPROM		"/dev/openprom"
 #define DEV_SUNMOUSE		"/dev/sunmouse"
 #define DEV_MICE		"/dev/input/mice"
-#define DEV_I2O			"/dev/i2o"
-#define DEV_CCISS		"/dev/cciss"
 #define DEV_FB			"/dev/fb"
 
-#define PROG_INSMOD		"/sbin/insmod"
 #define PROG_MODPROBE		"/sbin/modprobe"
 #define PROG_RMMOD		"/sbin/rmmod"
 #define PROG_CARDCTL		"/sbin/cardctl"
+#define PROG_UDEVINFO		"/usr/bin/udevinfo"
 
 #define KLOG_BOOT		"/var/log/boot.msg"
 #define ISAPNP_CONF		"/etc/isapnp.conf"
@@ -108,13 +102,12 @@ FILE *libhd_log;
  * Internal probing module numbers. Use mod_name_by_idx() outside of libhd.
  */
 enum mod_idx {
-  mod_none, mod_memory, mod_pci, mod_isapnp, mod_pnpdump, mod_cdrom,
-  mod_net, mod_floppy, mod_misc, mod_bios, mod_cpu, mod_monitor, mod_mouse,
-  mod_ide, mod_scsi, mod_serial, mod_usb, mod_adb, mod_modem, mod_parallel,
-  mod_isa, mod_dac960, mod_smart, mod_isdn, mod_kbd, mod_prom, mod_sbus,
-  mod_int, mod_braille, mod_xtra, mod_sys, mod_dasd, mod_i2o, mod_cciss,
-  mod_manual, mod_fb, mod_veth, mod_partition, mod_disk, mod_ataraid, mod_pppoe,
-  mod_pcmcia, mod_s390
+  mod_none, mod_memory, mod_pci, mod_isapnp, mod_pnpdump, mod_net,
+  mod_floppy, mod_misc, mod_bios, mod_cpu, mod_monitor, mod_mouse, mod_scsi,
+  mod_serial, mod_usb, mod_adb, mod_modem, mod_parallel, mod_isa, mod_isdn,
+  mod_kbd, mod_prom, mod_sbus, mod_int, mod_braille, mod_xtra, mod_sys,
+  mod_manual, mod_fb, mod_veth, mod_pppoe, mod_pcmcia, mod_s390,
+  mod_sysfs, mod_dsl, mod_block, mod_edd, mod_input
 };
 
 void *new_mem(size_t size);
@@ -128,8 +121,6 @@ void join_res_irq(hd_res_t **res1, hd_res_t *res2);
 void join_res_dma(hd_res_t **res1, hd_res_t *res2);
 hd_res_t *free_res_list(hd_res_t *res);
 hd_res_t *add_res_entry(hd_res_t **res, hd_res_t *new_res);
-hd_smbios_t *free_smbios_list(hd_smbios_t *sm);
-hd_smbios_t *add_smbios_entry(hd_smbios_t **sm, hd_smbios_t *new_sm);
 hd_t *add_hd_entry(hd_data_t *hd_data, unsigned line, unsigned count);
 misc_t *free_misc(misc_t *m);
 scsi_t *free_scsi(scsi_t *scsi, int free_all);
@@ -209,8 +200,6 @@ void hd_getdisksize(hd_data_t *hd_data, char *dev, int fd, hd_res_t **geo, hd_re
 str_list_t *hd_split(char del, char *str);
 char *hd_join(char *del, str_list_t *str);
 
-void hddb_add_info(hd_data_t *hd_data, hd_t *hd);
-
 int is_pnpinfo(ser_device_t *mi, int ofs);
 
 int is_pcmcia_ctrl(hd_data_t *hd_data, hd_t *hd);
@@ -224,6 +213,20 @@ void *hd_shm_add(hd_data_t *hd_data, void *ptr, unsigned len);
 int hd_is_shm_ptr(hd_data_t *hd_data, void *ptr);
 void hd_move_to_shm(hd_data_t *hd_data);
 
+void read_udevinfo(hd_data_t *hd_data);
+
+hd_t *hd_find_sysfs_id(hd_data_t *hd_data, char *id);
+int hd_attr_uint(struct sysfs_attribute *attr, uint64_t *u, int base);
+char *hd_attr_str(struct sysfs_attribute *attr);
+char *hd_sysfs_id(char *path);
+char *hd_sysfs_name2_dev(char *str);
+char *hd_sysfs_dev2_name(char *str);
+void hd_sysfs_driver_list(hd_data_t *hd_data);
+char *hd_sysfs_find_driver(hd_data_t *hd_data, char *sysfs_id, int exact);
+int hd_report_this(hd_data_t *hd_data, hd_t *hd);
+str_list_t *hd_module_list(hd_data_t *hd_data, unsigned id);
+
+
 #ifdef __cplusplus
 }
 #endif
