@@ -50,6 +50,7 @@ typedef struct {
   hd_id_t sub_vendor;
   hd_id_t sub_device;
   hd_id_t revision;
+  hd_id_t cu_model;
   char *serial;
   str_list_t *driver;
   char *requires;
@@ -58,7 +59,8 @@ typedef struct {
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static void hddb_init_pci(hd_data_t *hd_data);
-static driver_info_t *hd_pcidb(hd_data_t *hd_data, hd_t *hd);
+static hddb_pci_t *parse_pcimap(str_list_t *file);
+static driver_info_t *hd_pcidb(hd_data_t *hd_data, hddb_pci_t *pci_db, hd_t *hd);
 static void hddb_init_external(hd_data_t *hd_data);
 
 static line_t *parse_line(char *str);
@@ -83,6 +85,7 @@ static driver_info_t *monitor_driver(hd_data_t *hd_data, hd_t *hd);
 /* static int chk_free_biosmem(hd_data_t *hd_data, unsigned addr, unsigned len); */
 /* static isdn_parm_t *new_isdn_parm(isdn_parm_t **ip); */
 static driver_info_t *isdn_driver(hd_data_t *hd_data, hd_t *hd, cdb_isdn_card *cic);
+static driver_info_t *dsl_driver(hd_data_t *hd_data, hd_t *hd, cdb_isdn_card *cic);
 #endif
 
 static hd_res_t *get_res(hd_t *h, enum resource_types t, unsigned index);
@@ -90,32 +93,47 @@ static driver_info_t *reorder_x11(driver_info_t *di0, char *info);
 static void expand_driver_info(hd_data_t *hd_data, hd_t *hd);
 static char *module_cmd(hd_t *hd, char *cmd);
 
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 void hddb_init_pci(hd_data_t *hd_data)
 {
-  str_list_t *sl0 = NULL, *sl;
+  str_list_t *sl = NULL;
   char *s = NULL;
   struct utsname ubuf;
-  int len;
-  char buf[64];
-  unsigned u0, u1, u2, u3, u4, u5;
-  hddb_pci_t *p;
 
-  if(hd_data->hddb_pci) return;
+  if(!hd_data->hddb_pci) {
+    if(!uname(&ubuf)) {
+      str_printf(&s, 0, "/lib/modules/%s/modules.pcimap", ubuf.release);
+      sl = read_file(s, 0, 0);
+      s = free_mem(s);
+    }
 
-  /* during installation, the file is in /etc */
-  sl0 = read_file("/etc/modules.pcimap", 1, 0);
-  if(!sl0 && !uname(&ubuf)) {
-    str_printf(&s, 0, "/lib/modules/%s/modules.pcimap", ubuf.release);
-    sl0 = read_file(s, 1, 0);
-    s = free_mem(s);
+    hd_data->hddb_pci = parse_pcimap(sl);
+
+    sl = free_str_list(sl);
   }
 
-  for(len = 1, sl = sl0; sl; sl = sl->next) len++;
+  if(!hd_data->hddb_pci_hm) {
+    sl = read_file("/etc/hotplug/pci.handmap", 0, 0);
+    hd_data->hddb_pci_hm = parse_pcimap(sl);
+    sl = free_str_list(sl);
+  }
+}
 
-  hd_data->hddb_pci = new_mem(len * sizeof *hd_data->hddb_pci);
 
-  for(p = hd_data->hddb_pci, sl = sl0; sl; sl = sl->next) {
+hddb_pci_t *parse_pcimap(str_list_t *file)
+{
+  str_list_t *sl;
+  unsigned len;
+  hddb_pci_t *pci, *p;
+  char buf[64];
+  unsigned u0, u1, u2, u3, u4, u5;
+
+  for(len = 1, sl = file; sl; sl = sl->next) len++;
+
+  pci = new_mem(len * sizeof *pci);
+
+  for(p = pci, sl = file; sl; sl = sl->next) {
     if(sscanf(sl->str, "%63s %x %x %x %x %x %x", buf, &u0, &u1, &u2, &u3, &u4, &u5) == 7) {
       p->module = new_str(buf);
       p->vendor = u0;
@@ -129,27 +147,27 @@ void hddb_init_pci(hd_data_t *hd_data)
     }
   }
 
-  free_str_list(sl0);
-
 #if 0
-  for(p = hd_data->hddb_pci; p->module; p++) {
+  fprintf(stderr, "---  pcimap  ---\n");
+  for(p = pci; p->module; p++) {
     fprintf(stderr, "%s, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x\n",
       p->module, p->vendor, p->device, p->subvendor, p->subdevice,
       p->pciclass, p->classmask
     );
   }
 #endif
+
+  return pci;
 }
 
 
-driver_info_t *hd_pcidb(hd_data_t *hd_data, hd_t *hd)
+driver_info_t *hd_pcidb(hd_data_t *hd_data, hddb_pci_t *pci_db, hd_t *hd)
 {
   unsigned vendor, device, subvendor, subdevice, pciclass;
   driver_info_t *di = NULL, *di0 = NULL;
-  hddb_pci_t *p;
   pci_t *pci;
 
-  if(!(p = hd_data->hddb_pci)) return NULL;
+  if(!pci_db) return NULL;
 
   if(ID_TAG(hd->vendor.id) != TAG_PCI) return NULL;
 
@@ -167,13 +185,13 @@ driver_info_t *hd_pcidb(hd_data_t *hd_data, hd_t *hd)
     pciclass = (pci->base_class << 16) + ((pci->sub_class & 0xff) << 8) + (pci->prog_if & 0xff);
   }
 
-  for(; p->module; p++) {
+  for(; pci_db->module; pci_db++) {
     if(
-      (p->vendor == 0xffffffff || p->vendor == vendor) &&
-      (p->device == 0xffffffff || p->device == device) &&
-      (p->subvendor == 0xffffffff || p->subvendor == subvendor) &&
-      (p->subdevice == 0xffffffff || p->subdevice == subdevice) &&
-      !((p->pciclass ^ pciclass) & p->classmask)
+      (pci_db->vendor == 0xffffffff || pci_db->vendor == vendor) &&
+      (pci_db->device == 0xffffffff || pci_db->device == device) &&
+      (pci_db->subvendor == 0xffffffff || pci_db->subvendor == subvendor) &&
+      (pci_db->subdevice == 0xffffffff || pci_db->subdevice == subdevice) &&
+      !((pci_db->pciclass ^ pciclass) & pci_db->classmask)
     ) {
       if(di) {
         di = di->next = new_mem(sizeof *di);
@@ -183,7 +201,7 @@ driver_info_t *hd_pcidb(hd_data_t *hd_data, hd_t *hd)
       }
       di->any.type = di_module;
       di->module.modprobe = 1;
-      add_str_list(&di->any.hddb0, p->module);
+      add_str_list(&di->any.hddb0, pci_db->module);
     }
   }
 
@@ -1020,6 +1038,9 @@ int compare_ids(hddb2_data_t *hddb, hddb_search_t *hs, hddb_entry_mask_t mask, u
         case he_rev_id:
           id = hs->revision.id;
           break;
+	case he_detail_ccw_data_cu_model:
+	  id = hs->cu_model.id;
+	  break;
 
         default:
           ok = 0;
@@ -1213,6 +1234,10 @@ void complete_ids(
         case he_rev_id:
           id = &hs->revision.id;
           break;
+	
+	case he_detail_ccw_data_cu_model:
+	  id = &hs->cu_model.id;
+	  break;
 
         default:
           ok = 0;
@@ -1528,6 +1553,16 @@ void hddb_add_info(hd_data_t *hd_data, hd_t *hd)
     hs.key |= 1 << he_rev_name;
   }
 
+  if(hd->serial) {
+    hs.serial = hd->serial;
+    hs.key |= 1 << he_serial;
+  }
+
+  if(hd->detail && hd->detail->ccw.data) {
+    hs.cu_model.id=hd->detail->ccw.data->cu_model;
+    hs.key |= 1 << he_detail_ccw_data_cu_model;
+  }
+
   hddb_search(hd_data, &hs, 0);
 
   if((hs.value & (1 << he_bus_id))) {
@@ -1607,6 +1642,11 @@ void hddb_add_info(hd_data_t *hd_data, hd_t *hd)
     hd->sub_device.name = new_str(hs.sub_device.name);
   }
 
+  if((hs.value & (1 << he_detail_ccw_data_cu_model))) {
+    if(hd->detail && hd->detail->ccw.data)
+      hd->detail->ccw.data->cu_model=hs.cu_model.id;
+  }
+
   /* look for sub vendor again */
 
   if(!hd->sub_vendor.name && hd->sub_vendor.id) {
@@ -1658,8 +1698,19 @@ void hddb_add_info(hd_data_t *hd_data, hd_t *hd)
     }
     free_mem(cic);
   }
+  if (!new_driver_info && ((cic = get_dsl_info(hd)))) {
+    new_driver_info = dsl_driver(hd_data, hd, cic);
+    if(!hd->model && cic->lname && *cic->lname) {
+      hd->model = new_str(cic->lname);
+    }
+    free_mem(cic);
+  }
 #endif
 
+  if(!new_driver_info) {
+    new_driver_info = hd_pcidb(hd_data, hd_data->hddb_pci_hm, hd);
+  }
+
   if(!new_driver_info && (hs.value & (1 << he_driver))) {
     new_driver_info = hddb_to_device_driver(hd_data, &hs);
   }
@@ -1692,7 +1743,7 @@ void hddb_add_info(hd_data_t *hd_data, hd_t *hd)
   }
 
   if(!new_driver_info) {
-    new_driver_info = hd_pcidb(hd_data, hd);
+    new_driver_info = hd_pcidb(hd_data, hd_data->hddb_pci, hd);
   }
 
   if(new_driver_info) {
@@ -2253,6 +2304,68 @@ driver_info_t *isdn_driver(hd_data_t *hd_data, hd_t *hd, cdb_isdn_card *cic)
   return di0;
 }
 
+driver_info_t *dsl_driver(hd_data_t *hd_data, hd_t *hd, cdb_isdn_card *cic)
+{
+  driver_info_t *di0, *di;
+  cdb_isdn_vario *civ;
+  int drv;
+  str_list_t *sl, *sl0;
+
+  if(!cic) return NULL;
+
+  di0 = new_mem(sizeof *di0);
+
+  drv = cic->vario;
+  di = NULL;
+
+  while((civ = hd_cdbisdn_get_vario(drv))) {
+    drv = civ->next_vario;
+    if (di) {
+      di->next = new_mem(sizeof *di);
+      di = di->next;
+    } else {
+      di = di0;
+    }
+    di->dsl.type = di_dsl;
+    if(civ->interface && *civ->interface) {
+      if (!strcmp(civ->interface, "CAPI20")) {
+        di->dsl.mode = new_str("capiadsl");
+        if(civ->mod_name && *civ->mod_name)
+          di->dsl.name = new_str(civ->mod_name);
+        else
+          di->dsl.name = new_str("unknown");
+      } else if (!strcmp(civ->interface, "pppoe")) {
+        di->dsl.mode = new_str("pppoe");
+        if(civ->mod_name && *civ->mod_name)
+          di->dsl.name = new_str(civ->mod_name);
+        else
+          di->dsl.name = new_str("none");
+      } else {
+        di->dsl.mode = new_str("unknown");
+        di->dsl.name = new_str("unknown");
+      }
+    } else {
+      di->dsl.mode = new_str("unknown");
+      di->dsl.name = new_str("unknown");
+    }
+
+    if(civ->need_pkg && *civ->need_pkg) {
+      sl0 = hd_split(',', (char *) civ->need_pkg);
+      for(sl = sl0; sl; sl = sl->next) {
+        if(!search_str_list(hd->requires, sl->str)) {
+          add_str_list(&hd->requires, sl->str);
+        }
+      }
+      free_str_list(sl0);
+    }
+
+    if(hd->bus.id == bus_pci) continue;
+  }
+  if(!di) di0 = free_mem(di0);
+
+  return di0;
+}
+
 #endif		/* WITH_ISDN */
 
 
@@ -2387,6 +2500,13 @@ void expand_driver_info(hd_data_t *hd_data, hd_t *hd)
 
       case di_mouse:
         di->mouse.buttons = di->mouse.wheels = -1;
+        if(
+          hd->vendor.id == MAKE_ID(TAG_SPECIAL, 0x0210) &&
+          ID_TAG(hd->device.id) == TAG_SPECIAL
+        ) {
+          di->mouse.wheels = ID_VALUE(hd->device.id) >> 4;
+          di->mouse.buttons = ID_VALUE(hd->device.id) & 15;
+        }
         for(i = 0, sl = di->mouse.hddb0; sl; sl = sl->next, i++) {
           if(i == 0) {
             di->mouse.xf86 = new_str(sl->str);
