@@ -3,6 +3,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/pci.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include "hd.h"
 #include "hd_int.h"
@@ -25,12 +27,16 @@ static void int_media_check(hd_data_t *hd_data);
 static int contains_word(char *str, char *str2);
 static int is_zip(hd_t *hd);
 static void int_floppy(hd_data_t *hd_data);
-static void int_fix_ide_scsi(hd_data_t *hd_data);
 static void int_fix_usb_scsi(hd_data_t *hd_data);
 static void int_mouse(hd_data_t *hd_data);
 static void new_id(hd_data_t *hd_data, hd_t *hd);
 static void int_modem(hd_data_t *hd_data);
 static void int_wlan(hd_data_t *hd_data);
+static void int_udev(hd_data_t *hd_data);
+static void int_devicenames(hd_data_t *hd_data);
+#if defined(__i386__) || defined (__x86_64__)
+static void int_softraid(hd_data_t *hd_data);
+#endif
 
 void hd_scan_int(hd_data_t *hd_data)
 {
@@ -43,9 +49,6 @@ void hd_scan_int(hd_data_t *hd_data)
   /* some clean-up */
   remove_hd_entries(hd_data);
 
-  PROGRESS(1, 0, "idescsi");
-  int_fix_ide_scsi(hd_data);
-
   PROGRESS(2, 0, "cdrom");
   int_cdrom(hd_data);
 
@@ -63,14 +66,14 @@ void hd_scan_int(hd_data_t *hd_data)
   PROGRESS(6, 0, "mouse");
   int_mouse(hd_data);
 
-  PROGRESS(7, 0, "usbscsi");
-  int_fix_usb_scsi(hd_data);
-
-  PROGRESS(8, 0, "hdb");
+  PROGRESS(7, 0, "hdb");
   for(hd = hd_data->hd; hd; hd = hd->next) {
     hddb_add_info(hd_data, hd);
   }
 
+  PROGRESS(8, 0, "usbscsi");
+  int_fix_usb_scsi(hd_data);
+
   PROGRESS(9, 0, "hotplug");
   int_hotplug(hd_data);
 
@@ -79,6 +82,17 @@ void hd_scan_int(hd_data_t *hd_data)
 
   PROGRESS(11, 0, "wlan");
   int_wlan(hd_data);
+
+  PROGRESS(12, 0, "udev");
+  int_udev(hd_data);
+
+  PROGRESS(13, 0, "device names");
+  int_devicenames(hd_data);
+
+#if defined(__i386__) || defined (__x86_64__)
+  PROGRESS(14, 0, "soft raid");
+  int_softraid(hd_data);
+#endif
 }
 
 /*
@@ -279,6 +293,9 @@ void int_bios(hd_data_t *hd_data)
   int sctrl_len, i, j;
   int bios_id, list_sorted;
 
+  /* don't do anything if there is useful edd info */
+  if(hd_data->flags.edd_used) return;
+
   for(i = 0, hd = hd_data->hd; hd; hd = hd->next) {
     if(
       hd->base_class.id == bc_storage_device &&
@@ -457,6 +474,7 @@ void int_media_check(hd_data_t *hd_data)
   int i, j = 0;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(!hd_report_this(hd_data, hd)) continue;
     if(
       hd->base_class.id == bc_storage_device &&
       (
@@ -582,62 +600,6 @@ void int_floppy(hd_data_t *hd_data)
 }
 
 
-#define COPY_ENTRY(a) if(hd_ide->a) { free_mem(hd_scsi->a); hd_scsi->a = new_str(hd_ide->a); }
-/*
- * Remove ide entries that are handled by ide-scsi.
- */
-void int_fix_ide_scsi(hd_data_t *hd_data)
-{
-  hd_t *hd_scsi, *hd_ide;
-
-  for(hd_scsi = hd_ide = hd_data->hd; hd_scsi; hd_scsi = hd_scsi->next) {
-    if(
-      hd_scsi->bus.id == bus_scsi &&
-      hd_scsi->driver &&
-      !strcmp(hd_scsi->driver, "ide-scsi")
-    ) {
-      for(; hd_ide ; hd_ide = hd_ide->next) {
-        if(
-          hd_ide->bus.id == bus_ide &&
-          hd_ide->driver &&
-          !strcmp(hd_ide->driver, "ide-scsi")
-        ) {
-          // FIXME: we need a proper solution for this!
-          if(hd_ide->status.configured != status_no) {
-            hd_ide->tag.remove = 1;
-          }
-          hd_ide->status.available = status_no;
-
-          COPY_ENTRY(vendor.name);
-          COPY_ENTRY(device.name);
-          COPY_ENTRY(sub_device.name);
-          COPY_ENTRY(sub_vendor.name);
-          COPY_ENTRY(revision.name);
-          COPY_ENTRY(serial);
-
-          hd_scsi->is.notready &= hd_ide->is.notready;
-          hd_scsi->is.manual |= hd_ide->is.manual;
-          hd_scsi->is.softraiddisk |= hd_ide->is.softraiddisk;
-          hd_scsi->is.zip |= hd_ide->is.zip;
-          hd_scsi->is.cdr |= hd_ide->is.cdr;
-
-          free_mem(hd_scsi->unix_dev_name2);
-          hd_scsi->unix_dev_name2 = new_str(hd_ide->unix_dev_name);
-
-          new_id(hd_data, hd_scsi);
-
-          hd_ide = hd_ide->next;
-          break;
-        }
-      }
-    }
-  }
-
-  remove_tagged_hd_entries(hd_data);
-}
-#undef COPY_ENTRY
-
-
 #define COPY_ENTRY(a) if(hd_scsi->a) { free_mem(hd_usb->a); hd_usb->a = new_str(hd_scsi->a); }
 /*
  * Remove usb entries that are handled by usb-storage.
@@ -645,14 +607,47 @@ void int_fix_ide_scsi(hd_data_t *hd_data)
 void int_fix_usb_scsi(hd_data_t *hd_data)
 {
   hd_t *hd_scsi, *hd_usb;
-  usb_t *usb;
 
+  for(hd_usb = hd_data->hd; hd_usb; hd_usb= hd_usb->next) {
+    if(
+      hd_usb->bus.id == bus_usb &&
+      hd_usb->sysfs_id &&
+      search_str_list(hd_usb->drivers, "usb-storage")
+    ) {
+      for(hd_scsi = hd_data->hd; hd_scsi; hd_scsi = hd_scsi->next) {
+        if(
+          hd_scsi->bus.id == bus_scsi &&
+          hd_scsi->sysfs_device_link &&
+          search_str_list(hd_scsi->drivers, "usb-storage")
+        ) {
+          if(!strncmp(hd_scsi->sysfs_device_link, hd_usb->sysfs_id, strlen(hd_usb->sysfs_id))) {
+            hd_set_hw_class(hd_scsi, hw_usb);
+
+            free_mem(hd_scsi->unique_id);
+            hd_scsi->unique_id = hd_usb->unique_id;
+            hd_usb->unique_id = NULL;
+
+            add_res_entry(&hd_scsi->res, hd_usb->res);
+            hd_usb->res = NULL;
+
+            new_id(hd_data, hd_scsi);
+
+            hd_usb->tag.remove = 1;
+          }
+        }
+      }
+    }
+
+  }
+
+
+
+#if 0
   for(hd_scsi = hd_data->hd; hd_scsi; hd_scsi = hd_scsi->next) {
     if(
       hd_scsi->bus.id == bus_scsi &&
-      hd_scsi->driver &&
       hd_scsi->usb_guid &&
-      !strcmp(hd_scsi->driver, "usb-storage")
+      search_str_list(hd_scsi->drivers, "usb-storage")
     ) {
       for(hd_usb = hd_data->hd; hd_usb ; hd_usb = hd_usb->next) {
         if(
@@ -679,7 +674,7 @@ void int_fix_usb_scsi(hd_data_t *hd_data)
           COPY_ENTRY(prog_if.name);
           COPY_ENTRY(unix_dev_name);
           COPY_ENTRY(model);
-          COPY_ENTRY(driver);
+          // ###### FIXME?: COPY_ENTRY(driver)
 
           hd_usb->vendor.id = hd_scsi->vendor.id;
           COPY_ENTRY(vendor.name);
@@ -704,6 +699,9 @@ void int_fix_usb_scsi(hd_data_t *hd_data)
       }
     }
   }
+#endif
+
+
 
   remove_tagged_hd_entries(hd_data);
 }
@@ -773,6 +771,8 @@ void int_modem(hd_data_t *hd_data)
 {
   hd_t *hd;
   char *s;
+  hd_dev_num_t dev_num = { type: 'c', range: 1 };
+  unsigned cnt4 = 0;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
     if(
@@ -781,18 +781,33 @@ void int_modem(hd_data_t *hd_data)
       s = NULL;
       switch(hd->sub_class.id) {
         case sc_mod_win1:
-          s = "/dev/ham";
+          s = new_str("/dev/ham");
+          dev_num.major = 240;
+          dev_num.minor = 1;
           break;
         case sc_mod_win2:
-          s = "/dev/536ep";
+          s = new_str("/dev/536ep");
+          dev_num.major = 240;
+          dev_num.minor = 1;
           break;
         case sc_mod_win3:
-          s = "/dev/ttyLT0";
+          s = new_str("/dev/ttyLT0");
+          dev_num.major = 62;
+          dev_num.minor = 64;
+          break;
+        case sc_mod_win4:
+          if(cnt4 < 4) {
+            str_printf(&s, 0, "/dev/ttySL%u", cnt4);
+            dev_num.major = 212;
+            dev_num.minor = cnt4++;
+          }
           break;
       }
       if(s) {
         free_mem(hd->unix_dev_name);
-        hd->unix_dev_name = new_str(s);
+        hd->unix_dev_name = s;
+        s = NULL;
+        hd->unix_dev_num = dev_num;
       }
     }
   }
@@ -830,9 +845,25 @@ void int_wlan(hd_data_t *hd_data)
     "prism2_pci",
     "prism2_plx",
     "prism2_usb",
+    "prism54",
     "ray_cs",
     "wavelan",
-    "wavelan_cs"
+    "wavelan_cs",
+    "ipw2100",
+    "acx100_pci",
+    "atmel",
+    "atmel_cs",
+    "atmel_pci",
+    "at76c503-i3861",
+    "at76c503-i3863",
+    "at76c503-rfmd-acc",
+    "at76c503-rfmd",
+    "at76c503",
+    "at76c505-rfmd",
+    "at76c505-rfmd2958",
+    "usbdfu",
+    "wl3501_cs",
+    "ipw2100"
   };
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
@@ -857,3 +888,120 @@ void int_wlan(hd_data_t *hd_data)
   }
 }
 
+
+/*
+ * Add udev info.
+ */
+void int_udev(hd_data_t *hd_data)
+{
+  hd_udevinfo_t *ui;
+  hd_t *hd;
+  char *s = NULL;
+  str_list_t *sl;
+
+  if(!hd_data->udevinfo) read_udevinfo(hd_data);
+
+  if(!hd_data->udevinfo) return;
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(!hd->sysfs_id) continue;
+
+    for(ui = hd_data->udevinfo; ui; ui = ui->next) {
+      if(ui->name && !strcmp(ui->sysfs, hd->sysfs_id)) {
+        hd->unix_dev_names = free_str_list(hd->unix_dev_names);
+        hd->unix_dev_name = free_mem(hd->unix_dev_name);
+        str_printf(&s, 0, "/dev/%s", ui->name);
+        add_str_list(&hd->unix_dev_names, s);
+        for(sl = ui->links; sl; sl = sl->next) {
+          str_printf(&s, 0, "/dev/%s", sl->str);
+          add_str_list(&hd->unix_dev_names, s);
+        }
+        s = free_mem(s);
+
+        sl = hd->unix_dev_names;
+
+        if(hd_data->flags.udev) {
+          /* use first link as canonical device name */
+          if(ui->links) sl = sl->next;
+        }
+
+        hd->unix_dev_name = new_str(sl->str);
+
+        break;
+      }
+    }
+  }
+}
+
+
+/*
+ * If hd->unix_dev_name is not in hd->unix_dev_names, add it.
+ */
+void int_devicenames(hd_data_t *hd_data)
+{
+  hd_t *hd;
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(
+      hd->unix_dev_name &&
+      !search_str_list(hd->unix_dev_names, hd->unix_dev_name)
+    ) {
+      add_str_list(&hd->unix_dev_names, hd->unix_dev_name);
+    }
+  }
+}
+
+
+#if defined(__i386__) || defined (__x86_64__)
+/*
+ * Tag ide soft raid disks.
+ */
+void int_softraid(hd_data_t *hd_data)
+{
+  hd_t *hd;
+  str_list_t *raid, *sl, *raid_sysfs = NULL, *sl1;
+  size_t len;
+  char *s;
+
+  if(hd_data->flags.fast) return;
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(
+      hd->base_class.id == bc_storage_device &&
+      hd->status.available != status_no
+    ) break;
+  }
+
+  /* no disks -> no check necessary */
+  if(!hd) return;
+  
+  raid = read_file("| /sbin/raiddetect -s 2>/dev/null", 0, 0);
+
+  for(sl = raid; sl; sl = sl->next) {
+    s = hd_sysfs_id(sl->str);
+    if(s) {
+      sl1 = add_str_list(&raid_sysfs, s);
+      len = strlen(sl1->str);
+      if(len) sl1->str[len - 1] = 0;
+    }
+  }
+
+  free_str_list(raid);
+
+  ADD2LOG("----- soft raid devices -----\n");
+  for(sl = raid_sysfs; sl; sl = sl->next) {
+    ADD2LOG("  %s\n", sl->str);
+  }
+  ADD2LOG("----- soft raid devices end -----\n");
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(search_str_list(raid_sysfs, hd->sysfs_id)) {
+      hd->is.softraiddisk = 1;
+    }
+  }
+
+  free_str_list(raid_sysfs);
+
+}
+#endif
+
