@@ -82,6 +82,10 @@ static hash_t hw_items[] = {
   { hw_zip,           "zip"                 },
   { hw_pppoe,         "pppoe"               },
   { hw_wlan,          "wlan card"           },
+  { hw_dsl,           "DSL adapter"         },
+  { hw_block,         "block device"        },
+  { hw_tape,          "tape"                },
+  { hw_vbe,           "vesa bios"           },
   { hw_unknown,       "unknown"             },
   { 0,                NULL                  }
 };
@@ -119,10 +123,12 @@ typedef enum {
   hwdi_prog_if, hwdi_dev, hwdi_vend, hwdi_sub_dev, hwdi_sub_vend, hwdi_rev,
   hwdi_compat_dev, hwdi_compat_vend, hwdi_dev_name, hwdi_vend_name,
   hwdi_sub_dev_name, hwdi_sub_vend_name, hwdi_rev_name, hwdi_serial,
-  hwdi_unix_dev_name, hwdi_rom_id, hwdi_broken, hwdi_usb_guid, hwdi_res_mem,
-  hwdi_res_phys_mem, hwdi_res_io, hwdi_res_irq, hwdi_res_dma, hwdi_res_size,
-  hwdi_res_baud, hwdi_res_cache, hwdi_res_disk_geo, hwdi_res_monitor,
-  hwdi_res_framebuffer, hwdi_features, hwdi_hotplug, hwdi_class_list
+  hwdi_unix_dev_name, hwdi_unix_dev_name2, hwdi_unix_dev_names, hwdi_rom_id,
+  hwdi_broken, hwdi_usb_guid, hwdi_res_mem, hwdi_res_phys_mem, hwdi_res_io,
+  hwdi_res_irq, hwdi_res_dma, hwdi_res_size, hwdi_res_baud, hwdi_res_cache,
+  hwdi_res_disk_geo, hwdi_res_monitor, hwdi_res_framebuffer, hwdi_features,
+  hwdi_hotplug, hwdi_class_list, hwdi_drivers, hwdi_sysfs_id,
+  hwdi_sysfs_busid, hwdi_sysfs_link
 } hw_hd_items_t;
 
 static hash_t hw_ids_hd_items[] = {
@@ -146,6 +152,8 @@ static hash_t hw_ids_hd_items[] = {
   { hwdi_rev_name,        "RevisionName"     },
   { hwdi_serial,          "Serial"           },
   { hwdi_unix_dev_name,   "UnixDevice"       },
+  { hwdi_unix_dev_name2,  "UnixDeviceAlt"    },
+  { hwdi_unix_dev_names,  "UnixDeviceList"   },
   { hwdi_rom_id,          "ROMID"            },
   { hwdi_broken,          "Broken"           },
   { hwdi_usb_guid,        "USBGUID"          },
@@ -163,6 +171,10 @@ static hash_t hw_ids_hd_items[] = {
   { hwdi_features,        "Features"         },
   { hwdi_hotplug,         "Hotplug"          },
   { hwdi_class_list,      "HWClassList"      },
+  { hwdi_drivers,         "Drivers"          },
+  { hwdi_sysfs_id,        "SysfsID"          },
+  { hwdi_sysfs_busid,     "SysfsBusID"       },
+  { hwdi_sysfs_link,      "SysfsLink"        },
   { 0,                    NULL               }
 };
 
@@ -887,6 +899,30 @@ void manual2hd(hd_data_t *hd_data, hd_manual_t *entry, hd_t *hd)
         hd->unix_dev_name = new_str(sl2->str);
         break;
 
+      case hwdi_unix_dev_name2:
+        hd->unix_dev_name2 = new_str(sl2->str);
+        break;
+
+      case hwdi_unix_dev_names:
+        hd->unix_dev_names = hd_split(' ', sl2->str);
+        break;
+
+      case hwdi_drivers:
+        hd->drivers = hd_split('|', sl2->str);
+        break;
+
+      case hwdi_sysfs_id:
+        hd->sysfs_id = new_str(sl2->str);
+        break;
+
+      case hwdi_sysfs_busid:
+        hd->sysfs_bus_id = new_str(sl2->str);
+        break;
+
+      case hwdi_sysfs_link:
+        hd->sysfs_device_link = new_str(sl2->str);
+        break;
+
       case hwdi_rom_id:
         hd->rom_id = new_str(sl2->str);
         break;
@@ -1244,6 +1280,40 @@ void hd2manual(hd_t *hd, hd_manual_t *entry)
     add_str_list(&entry->value, hd->unix_dev_name);
   }
 
+  if(hd->unix_dev_name2) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_unix_dev_name2));
+    add_str_list(&entry->value, hd->unix_dev_name2);
+  }
+
+  if(hd->unix_dev_names) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_unix_dev_names));
+    s = free_mem(s);
+    s = hd_join(" ", hd->unix_dev_names);
+    add_str_list(&entry->value, s);
+  }
+
+  if(hd->drivers) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_drivers));
+    s = free_mem(s);
+    s = hd_join("|", hd->drivers);
+    add_str_list(&entry->value, s);
+  }
+
+  if(hd->sysfs_id) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_sysfs_id));
+    add_str_list(&entry->value, hd->sysfs_id);
+  }
+
+  if(hd->sysfs_bus_id) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_sysfs_busid));
+    add_str_list(&entry->value, hd->sysfs_bus_id);
+  }
+
+  if(hd->sysfs_device_link) {
+    add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_sysfs_link));
+    add_str_list(&entry->value, hd->sysfs_device_link);
+  }
+
   if(hd->rom_id) {
     add_str_list(&entry->key, key2value(hw_ids_hd_items, hwdi_rom_id));
     add_str_list(&entry->value, hd->rom_id);
@@ -1450,6 +1520,8 @@ int hd_write_config(hd_data_t *hd_data, hd_t *hd)
   int err = 0;
   hd_manual_t *entry;
 
+  if(!hd_report_this(hd_data, hd)) return 0;
+
   entry = new_mem(sizeof *entry);
 
   hd2manual(hd, entry);
